# -*- coding: utf-8 -*-
"""
地理特征聚类
Created on Thu Feb  1 22:09:50 2018

@author: jasonai
"""

import pandas as pd
from sklearn.cluster import KMeans

df = pd.read_excel('各省市区点评餐饮点数据.xlsx', index_col=[0,1,2])
df.fillna(0, inplace=True)

df2 = df.apply(lambda x: x / df.sum(axis=1))
df2.dropna(inplace=True)

#设定不同k值以运算
for k in range(5,6):
    clf = KMeans(n_clusters=k, max_iter=1000) #设定k ！！！这里就是调用KMeans算法
    s = clf.fit(df2) #加载数据集合
    numSamples=len(df2)
    centroids = clf.labels_  # 每个样本所属的簇
    #print(centroids) #显示中心点
    print(clf.inertia_)  #显示聚类效果

df2['类别'] = centroids


# -*- coding: utf-8 -*-
"""
Created on Mon Feb 26 15:43:50 2018

@author: Administrator
"""
import yaml
import numpy as np
import pandas as pd


# 分配统一和今麦郎的销量至康师傅的各商圈
def allocation_sale(df_ksf, df_tyjml, period):
    # 确保三甲公司五种规格的名称统一
    df_ksf = df_ksf.fillna(0)
    df_tyjml = df_tyjml.fillna(0)
    spec_list = [i+'.{}'.format(period-1) for i in ['容器面', '高价袋','中价面','平低价','干脆面']]
    df_ksf2 = df_ksf.loc[:, ["营业部","具体描述"]+spec_list]
    df_tyjml2 = df_tyjml.loc[:, ["营业部"]+spec_list].groupby("营业部").sum()
    df_depart = df_ksf2.groupby("营业部").sum() + df_tyjml2.drop('MT部')
    # 将MT部的销量分散在四个营业部中
    df_depart2 = df_tyjml2.loc['MT部'] * df_depart / df_depart.sum() + df_depart
    # 将康师傅其他MT部的销量分配到康师傅各商圈中
    df_ksf2 = df_ksf2.set_index(["营业部","具体描述"])
    df_ksf3 = df_tyjml.loc['康师傅其他', spec_list] * df_ksf2 / df_ksf2.sum() + df_ksf2
    df_ksf3 = df_ksf3.astype('float64')
    df_ksf_actual = df_ksf3.copy()
    # 额外计算整体市场
    df_ksf_actual['整体'] = df_ksf_actual.loc[:, spec_list].sum(axis=1)
    # 输出康师傅各商圈实际出货量
    df_ksf3.to_excel('KSF各商圈实际出货量.xlsx')
    # 将三大厂商的营业部出货数据拆分到KSF各商圈
    # 如果平低价为空，则将使用中价面和干脆面的量模糊代表中价面
    lowPrice = '平低价.{}'.format(period-1)
    if True in df_ksf3.groupby('营业部').sum()[lowPrice].isnull().values:
        df_ksf3.loc[:, lowPrice] = df_ksf3.loc[:, ['中价面.{}'.format(period-1), '干脆面.{}'.format(period-1)]].sum(axis=1)
    df_ksf4 = df_ksf3.apply(lambda x: x/x.groupby('营业部').sum())
    for index, d in enumerate(df_ksf["营业部"].drop_duplicates()):
        if not index:
            df_ksf5 = df_ksf4.xs(d, level='营业部') * df_depart2.loc[d]
        else:
            df_temp = df_ksf4.xs(d, level='营业部') * df_depart2.loc[d]
            df_ksf5 = pd.concat([df_ksf5, df_temp])
            
    # 重新获取营业部信息
    df_3c = pd.concat([df_ksf.loc[:, ["营业部","具体描述"]].set_index("具体描述"), df_ksf5], axis=1)
    # df_3c.to_excel('各商圈三甲市场量.xlsx')

    return df_3c, df_ksf_actual  # 三甲公司实际出货量, 康师傅实际出货量

# 校准矩阵1--包含0.5权重的销量通路点数和0.5权重的正负因子系数bias
# 输入DataFrame需包含三甲在各商圈出货量，各商圈通路点数
def calibration_matrix1(df_3c, df_channel, province, period, df_bias=[]):
    # 校准至0.7至1.2
    def ratio_calibration(x):
        # tanh函数
        tanh = lambda x: 2.0 / (1.0 + np.exp(-2*x)) - 1
        if x > 1: 
            # 控制在1.2以下
            y = 0.3 * tanh(0.5*x-0.5) + 1
        else:
            # 控制在0.6以上
            y = 0.5 * tanh(0.35*x-0.5) + 1
        return y

    # 选取指定年份下的规格别
    spec_list = [i+'.{}'.format(period-1) for i in ['容器面', '高价袋','中价面','平低价','干脆面']]
    ## 各商圈销量总和与通路总和的比值
    df_channel1 = df_3c.loc[:, spec_list].apply(lambda x: df_channel['总通路点数'] / x)
    # 各商圈销量总和与通路总和的比值
    sr_channel = df_channel['总通路点数'].sum() / df_3c.loc[:, spec_list].sum()
    df_channel1 = df_channel1 / sr_channel
    df_adjust1 = df_channel1.applymap(ratio_calibration)

    # 对预估与实际的比值（bias）进行校准,如果为空，则只采用df_adjust1
    if len(df_bias):
        try:
            df_bias = df_bias.loc[:, spec_list] # 首先尝试提取指定年份的规格别销量
        except:
            df_bias = df_bias.loc[:, ['容器面平均', '高价袋平均','中价面平均','平低价平均','干脆面平均']]
            df_bias.columns = spec_list
        df_adjust2 = df_bias.applymap(ratio_calibration)
        cal_mat1 = 0.5 * df_adjust1 +  0.5 * df_adjust2
    else:
        cal_mat1 = df_adjust1
    # 输出各商圈因子综合影响系数和通路影响系数
    #df_adjust1.to_excel('%s各商圈正负因子综合指数.xlsx' % province)
    #df_adjust2.to_excel('%s各商圈各通路综合指数.xlsx' % province)
    cal_mat1.to_excel('%s各商圈正负因子综合指数.xlsx' % province)
    return cal_mat1

# 获取尼尔森提供的三甲在总方便面市场中的市占
def calibration_AC(df_mutilComp, province, period):
    # 提取对应省份对应年份的物种规格别销量
    try:
        df_mutilComp2 = df_mutilComp.loc[province + '总和']['年销量.{}'.format(period-1)]
    except KeyError: # 省份名称切换
        df_mutilComp2 = df_mutilComp.loc[[x[0] for x in prov_mapp.items() 
        if province in x[1]][0] + '总和']['年销量.{}'.format(period-1)]
    # 将尼尔森的低平价面改为地评价
    # spec_list = [i+'.{}'.format(period-1) for i in ['容器面', '高价袋','中价面','平低价','干脆面']]
    spec_list_old = [i[:3]+'.{}'.format(period-1) for i in df_mutilComp2]
    df_mutilComp2.columns = spec_list_old
    cal_AC = df_mutilComp2.loc[['康师傅', '统一', '今麦郎'], :].sum()
    
    return cal_AC

# 获取尼尔森提供的三甲在总方便面市场中的市占
def calibration_manual(period, province):
    # 将规格别列名修改为对应年份的规格别
    df = pd.read_excel('Calibration_manual.xlsx')
    df.columns = [i+'.{}'.format(period-1) for i in df.columns]
    cal_man = df.loc[province]
    
    return cal_man

# 各商圈总方便面市场量预估
def market_total(df_3c, cal_mat1, cal_AC, cal_man):
    df_total = df_3c.reset_index().set_index(["营业部","具体描述"])
    df_total = df_total * cal_mat1 * cal_man / cal_AC
    df_total['整体'] = df_total.sum(axis=1)  
    return df_total
    

def market_estimate(province='湖北省', period=3):
    spec_list = [i+'.{}'.format(period-1) for i in ['容器面', '高价袋','中价面','平低价','干脆面']]
    # 获取康师傅各商圈实际出货，三甲各商圈实际出货
    df_3c, df_ksf2 = allocation_sale(df_ksf, df_tyjml, period)
    # 获取校准系数矩阵1
    cal_mat1 = calibration_matrix1(df_3c, df_channel, province, period, df_bias)
    # 获取三甲厂商各规格占总市场各规格的比例
    cal_AC = calibration_AC(df_mutilComp, province, period)
    # 获取经验人工校准数据
    cal_man = calibration_manual(period, province)
    # 预估各规格总市场量
    df_total = market_total(df_3c, cal_mat1, cal_AC, cal_man)
    # 计算营业部市占
    df_marketShare1 = df_ksf2.groupby('营业部').sum() / df_total.groupby('营业部').sum()
    # 计算各商圈市占
    df_ksf2_s = df_ksf2.reset_index(level='营业部', drop=True)
    df_total_s = df_total.reset_index(level='营业部', drop=True)
    df_ksf2_s.loc[province] = df_ksf2_s.sum()
    df_total_s.loc[province] = df_total_s.sum()
    df_marketShare2 = df_ksf2_s / df_total_s  
    # 计算人均包
    df_pop = df_channel.copy()
    df_pop.loc[province] = df_channel.sum()
    df_population = df_pop['人口'] # 获取各商圈人口信息
    # 将千箱转化为包
    df_capita = df_total_s * 24
    df_capita['容器面.{}'.format(period-1)] = df_capita['容器面.{}'.format(period-1)] / 2
    df_capita['整体'] = df_capita.loc[:, spec_list].sum(axis=1)
    df_capita2 = df_capita.apply(lambda x: x * 1000/ df_population) 
    df_capita2['整体'] = df_capita2.loc[:, spec_list].sum(axis=1)
    # 重新排序，将湖北省放最后
    df_capita2 = df_capita2.reindex(df_capita.index)
    df_total_s.loc[:, spec_list+['整体']].to_excel('{}各商圈总市场量（千箱）.xlsx'.format(province))
    df_capita.loc[:, spec_list+['整体']].to_excel('{}各商圈总市场量（千包）.xlsx'.format(province))
    df_marketShare1.loc[:, spec_list+['整体']].to_excel('{}各营业部市占.xlsx'.format(province))
    df_marketShare2.loc[:, spec_list+['整体']].to_excel('{}各商圈市占.xlsx'.format(province))
    df_capita2.loc[:, spec_list+['整体']].to_excel('{}各商圈人均包.xlsx'.format(province))


if __name__ == '__main__':
    df_ksf = pd.read_excel('湖北省KSF各商圈TT&MT出货.xlsx', sheetname='TT&MT')
    df_tyjml = pd.read_excel('统一和今麦郎出货量.xlsx', index_col="厂商")
    df_bias = pd.read_excel("湖北预估与实际的比值-最新.xlsx", index_col = '具体描述')
    df_dapart = pd.read_excel("营业部与商圈对应.xlsx", index_col = '具体描述')
    df_market = pd.read_excel("最新一年KSF各个部门的三甲市占.xlsx")
    df_channel = pd.read_excel("通路点数和人口.xlsx", index_col = '具体描述')
    df_mutilComp = pd.read_excel("各范围各厂商方便面.xlsx", index_col=[0,1], header=[0,1], sheetname='市占')
    prov_mapp = yaml.load(open('E:\CloudStation\OCM\省份名单对应表.yaml', encoding='utf-8'))
  
    market_estimate()



"""
df = df.applymap(lambda x: 1/x)
df.to_excel("湖北实际与预估的比值-最新.xlsx")
category = ['整体', '中高价面',	'高价面', '容器面','高价袋',	'中价面',	'平低价','干脆面']
for cate in category:
    # 计算三年平均
    df_channel[('平均', '{0}平均'.format(cate))] = df_channel.loc[:,[('2015年',cate),('2016年',cate), ('2017年',cate)]].mean(axis=1)
    # 计算通路点数/三年平均
    df_channel[('平均通路点数', '{0}通路'.format(cate))] = df_channel.loc[:,[('2015年',cate),('2016年',cate), ('2017年',cate)]].mean(axis=1)
df_channel.to_excel("业绩和通路点数2.xlsx")
"""

####################补充说明############################
## tanh曲线的绘制方法：
"""
import matplotlib.pyplot as plt  
import matplotlib as mpl  
mpl.rcParams['axes.unicode_minus']=False 
fig = plt.figure(figsize=(6,4))  
ax = fig.add_subplot(111)  
  
x = np.linspace(-10, 10)  
tanh = lambda x: 2.0 / (1.0 + np.exp(-2*x)) - 1    
  
plt.xlim(-11,11)  
plt.ylim(-1.1,1.1)  
  
ax.spines['top'].set_color('none')  
ax.spines['right'].set_color('none')  
  
ax.xaxis.set_ticks_position('bottom')  
ax.spines['bottom'].set_position(('data',0))  
ax.set_xticks([-10,-5,0,5,10])  
ax.yaxis.set_ticks_position('left')  
ax.spines['left'].set_position(('data',0))  
ax.set_yticks([-1,-0.5,0.5,1])  
yValue = 0.5 * tanh(0.35*x)  
plt.plot(2*x,yValue,label="Tanh", color = "red")  
plt.legend()  
plt.show()
"""




# -*- coding: utf-8 -*-
"""
Created on Mon Mar 12 13:46:03 2018

@author: Administrator
"""
import pandas as pd
import random

df = pd.read_excel('全国正负因子.xlsx', index_col=0, header=[0,1])

# 正因子
def zyz(n=8):
    return [random.uniform(0.1,0.4) for i in range(n)]

# 整体面的中间带
def zt_zjd(n=9):
    return [random.uniform(-0.2,0.2) for i in range(n)]

# 容器面和高价袋的中间带
def gjm_zjd(n=9):
    return [random.uniform(0,0.2) for i in range(n)]

# 低价面的中间带
def djm_zjd(n=9):
    return [random.uniform(-0.2,0) for i in range(n)]

# 负因子
def fyz_zjd(n=5):
    return [random.uniform(-0.2,-0.1) for i in range(n)]  

unit = int(len(df.columns) / 6)

prov_list = []
# 整体
for index, u in enumerate(range(unit)):
    df.iloc[:, 6*u] = zyz() + zt_zjd() + fyz_zjd() # 整体
    s1 = df.iloc[:, 6*u].sort_values(ascending=False).reset_index()
    
    df.iloc[:, 6*u+1] = zyz() + gjm_zjd() + fyz_zjd() # 容器面
    s2 = df.iloc[:, 6*u+1].sort_values(ascending=False).reset_index()
    
    df.iloc[:, 6*u+2] = zyz() + gjm_zjd() + fyz_zjd() # 高价袋
    s3 = df.iloc[:, 6*u+2].sort_values(ascending=False).reset_index()
    
    df.iloc[:, 6*u+3] = zyz() + djm_zjd() + fyz_zjd() # 中价面
    s4 = df.iloc[:, 6*u+3].sort_values(ascending=False).reset_index()
    
    df.iloc[:, 6*u+4] = zyz() + djm_zjd() + fyz_zjd() # 低平价面
    s5 = df.iloc[:, 6*u+4].sort_values(ascending=False).reset_index()
    
    df.iloc[:, 6*u+5] = zyz() + djm_zjd() + fyz_zjd() # 干脆面
    s6 = df.iloc[:, 6*u+5].sort_values(ascending=False).reset_index()
    
    df_all = pd.concat([s1,s2,s3,s4,s5,s6], axis=1)
    df_all.to_excel(df.columns[6*u][0]+'正负因子2.xlsx')



df.to_excel('全国正负因子-河南2.xlsx')
# df2.to_excel('ts2.xlsx')




import os, sys
import numpy as np
import pandas as pd


def merging_data(df, province, col_essential):
    featrue_series = df.loc[province].groupby('一级特征').first()['文件名']
    feature1_list = list(featrue_series.index)
    ftfile_list = list(featrue_series.values)
    df.fillna(0, inplace=True)
    sub_feature_index = list(df.columns).index('子特征1')
    feature_ori_index = list(df.columns).index('特征原名')
    for index, (feature1, feature1_filename) in enumerate(zip(feature1_list, ftfile_list)):
        # 从一级特征对应文件读取数据框
        # 判断工作目录中是否存在region合并化后的文件
        merging_filename = os.path.splitext(feature1_filename)[0] + '_region.xlsx'

        if os.path.exists(merging_filename):
            df1 = pd.read_excel(merging_filename, index_col=[0, 1, 2])
        else:
            # 解决直属县级市的问题
            # 判断文件类型，目前只考虑了'xlsx'和'csv'
            if feature1_filename.split('.')[-1] == 'csv':
                df1 = pd.read_csv(feature1_filename, engine='python')
            else:
                df1 = pd.read_excel(feature1_filename)

            # 解决省直属的问题
            if len(col_essential) > 2:
                df1[col_essential[2]] = np.where(df1[col_essential[2]].isnull(), df1[col_essential[1]], df1[col_essential[2]])
            df1 = df1.groupby(col_essential).sum()
            df1.to_excel(merging_filename)
        
        # 判断是否存在需通过子特征合并处理的特征，如存在则需先合并子特征并存于新的特征列
        df_s1 = df.loc[province][df.loc[province]['一级特征'] == feature1]
        for i in df_s1.values:
            if i[sub_feature_index]:
                df1[i[feature_ori_index]] = df1.loc[:, i[sub_feature_index:]].sum(axis=1)
        # 根据'特征原名'列提取各维度特征
        feature_list = list(df_s1['特征原名'])
        feature_change_list = list(df_s1['使用特征名'])
        df2 = df1.loc[:, feature_list]
        # 更改特征名
        df2.columns = feature_change_list
        if not index:
            df3 = df2
        else:
            df3 = df3.merge(df2, left_index=True, right_index=True, how='outer')

    # 使用0填充缺失值
    df3.fillna(0, inplace=True)
    df3.to_excel('%s_特征.xlsx' % province)


if __name__ == "__main__":
    feature_file = '湖北需求预估使用的特征.xlsx'
    province = '湖北'
    df = pd.read_excel(feature_file, index_col=[0])  # 将'省'列设为索引
    merging_data(df, province, col_essential)




import os, sys
import re
import time
import shutil
import numpy as np
import pandas as pd
from sklearn.model_selection import train_test_split
from sklearn.preprocessing import StandardScaler
from keras.models import Sequential
from keras.layers import Dense
# from keras.optimizers import SGD
from pmipy.DemandPredict.Visualization import SnsVisualization
import seaborn as sns
import matplotlib as mpl
import matplotlib.pyplot as plt
from pyplotz.pyplotz import PyplotZ
from pmipy.DemandPredict import adjustment_interpreter
from pmipy.DemandPredict import AnalyzeGrading

zhfont1 = mpl.font_manager.FontProperties(fname='C:\Windows\Fonts\simkai.ttf')  # 解决python可视化中文乱码的问题


class BusinessSalesPrediction(SnsVisualization):
    def __init__(self, work_dir, df, sales_file_proc, ocm_file, time_tag, sale_tag, sales_mapping_file, province):
        self.work_dir = work_dir
        self.time_tag = time_tag
        self.file_tag = os.path.splitext(sales_mapping_file)[0]
        self.df1 = df
        df2 = pd.read_excel(os.path.join(work_dir, ocm_file), index_col=2)
        self.df2 = df2.drop(['省', '市'], axis=1).T
        self.sales_mapping_file = sales_mapping_file
        self.province =province
        self.sales_file_proc = sales_file_proc
        self.sale_tag = sale_tag

    # 处理康师傅提供的各商圈销量数据，关键点是合并一个商圈多个经销商的销量数据
    def sum_mutil_dealer(self, sales_file_ori):
        """
        如要读取Excel多个sheet，则可以通过下列方法实现：
        xls = pd.ExcelFile('path_to_file.xls')
        df = pd.read_excel(xls, 'Sheet1')
        或者通过上下文管理器实现：
        with pd.ExcelFile('path_to_file.xls') as xls:
            df1 = pd.read_excel(xls, 'Sheet1')
            df2 = pd.read_excel(xls, 'Sheet2')
        """
        df = pd.read_excel(os.path.join(self.work_dir, sales_file_ori), skiprows=2)  # 忽略前两行
        df.drop(['省', '市', '区/县', '营业部', '现有经销商/物流公司名称', '客户形态', 
                 '现有经销商/物流公司代码'], axis=1, inplace=True)
        df.fillna(method='pad', inplace=True)  # excel原合并行缺失值填充
        # 由于20171201日客户又添加了商圈客户形态等级，在一下部分代码作了修改
        df.drop_duplicates(['具体描述', '整体', '高价面'], inplace=True)  # 先处理一商圈多区县的销量数据
        df2 = df.groupby(['具体描述']).sum()  # 合并一商圈多经销商销量数据
        # 计算各商圈历年'整体平均千箱'、'高价平均千箱'、'整体平均千元'、'高价平均千元'的平均数据
        df2['整体平均千箱'] = df2.loc[:, ['整体', '整体.1', '整体.2']].mean(axis=1)
        df2['高价平均千箱'] = df2.loc[:, ['高价面', '高价面.1', '高价面.2']].mean(axis=1)
        df2['整体平均千元'] = df2.loc[:, ['整体.3', '整体.4', '整体.5']].mean(axis=1)
        df2['高价平均千元'] = df2.loc[:, ['高价面.3', '高价面.4', '高价面.5']].mean(axis=1)
        # 提取平均销量数据及最近一年的销量平均数据
        df3 = df2.loc[:, ['整体.2', '整体平均千箱', '整体.5', '整体平均千元', '高价面.2', '高价平均千箱', '高价面.5',
                          '高价平均千元']]
        df3.rename(index=str, columns={"整体.2": "整体2017千箱", "整体.5": "整体2017千元", "高价面.2": "高价2017千箱",
                                       "高价面.5": "高价2017千元"}, inplace=True)
        # 对商圈等级进行排序
        # df3.sort_index(level='客户形态对应的商圈等级', inplace=True)
        df3.sort_index(level='具体描述', inplace=True)
        print('%s共有%d商圈' % (self.province, len(df3)))
        df3.to_excel(self.sales_file_proc)

    def ocm_pretreatment(self):
        # 部分商圈包含多个区县，因此需先合并这类商圈的OCM参数
        df = pd.DataFrame(self.df2.index, columns=['区县'], index=self.df2.index)
        for business in self.df1.index:
            # 包含“市辖区”字段的商圈有一个或多个区县，不包含的只有一个区县
            if ("、" in business) or ("，" in business):
                business_parse = re.split(r'[（、，）]', business)[1:-1]
                try:
                    df.loc[:, business] = self.df2.loc[:, business_parse].sum(axis=1)
                except Exception as err:
                    print(err)  # 如果报错，将输出OCM中缺失的区县
            else:
                if business in self.df2.columns:
                    df[business] = self.df2[business]
        df.set_index("区县", inplace=True)
        df = df.T
        df = pd.concat([df, self.df1[self.sale_tag]], axis=1, join='inner')
        df.index.set_names("商圈", inplace=True)
        df.to_excel(self.sales_mapping_file)
        return df

    # 传统神经网络
    def neural_network_model(self, data, step_list=[50], mutil_NN=False, test_size=0.3):
        mutil_tag = "MutilNN" if mutil_NN else "MonoNN"  # 判断单层还是多层神经网络
        dim = len(data.columns) - 1
        parameter = np.append(data.columns[:-1], ['bias'])
        # 数据标准化
        scaler = StandardScaler().fit(data.values)  # sklearn的标准化类
        mean = scaler.mean_  # 平均值，为后续还原真实数据准备
        std = np.sqrt(scaler.var_)  # 标准差，为后续还原真实数据准备

        # random_state：可以为整数、RandomState实例或None，默认为None
        # 若为None时，每次生成的数据都是随机，可能不一样,若为整数时，每次生成的数据都相同
        train_data, test_data = train_test_split(data, test_size=test_size)
        X_train, y_train = scaler.transform(train_data)[:, :-1], scaler.transform(train_data)[:, -1]
        X_test, y_test = scaler.transform(test_data)[:, :-1], scaler.transform(test_data)[:, -1]
        # sgd = SGD(lr=0.005)

        # 此函数用于判断预测值与真实值的偏差，范围在0.2以后表示较为理想
        def deviation(y_true, y_pred):
            return (y_pred - y_true) / y_true
        effective_learning = True  # 用于判断是否有有效学习
        # 创建文件夹
        if not os.path.exists(self.time_tag):
            os.mkdir(self.time_tag)
        # 探究训练迭代次数对参数权重的影响
        for index, time_step in enumerate(step_list):
            model = Sequential()
            if mutil_NN:
                model.add(Dense(dim * 4, input_dim=dim, activation='relu'))
                model.add(Dense(dim * 2, activation='relu'))
                model.add(Dense(1))
            else:
                model.add(Dense(1, input_dim=dim))

            model.compile(loss='mse', optimizer='sgd', metrics=[deviation])  # 目前暂不使用自己定义的sgd
            model.fit(X_train, y_train, batch_size=10, epochs=time_step)  # batch_size需优化
            score = model.evaluate(X_test, y_test, verbose=0)
            print('Test loss:', score[0])
            if score[0] > 0.3:  # 设置loss上限
                print("Test loss is over 0.3, the model is abandoned!")
                effective_learning = False
                shutil.rmtree(self.time_tag)
                break
            # 获取个参数的权重
            W, b = model.layers[0].get_weights()
            W = np.append(W, b)
            df_w = pd.DataFrame(W)
            df_w.columns = [time_step]
            df_w2 = df_w if not index else pd.concat([df_w2, df_w], axis=1)
            # 保存模型
            model.save(os.path.join(self.time_tag, "%s_model_%s_%s.h5" % (self.file_tag, mutil_tag, self.time_tag)))

            # 训练集真实数据及预测数据
            df_ytrain = train_data.iloc[:, -1:]
            y_train_pred = model.predict(X_train)
            df_ytrain['预估销量'] = y_train_pred * std[-1] + mean[-1]
            df_ytrain['training_time'] = time_step
            df_ytrain['预估与实际比值'] = df_ytrain['预估销量'] / df_ytrain[self.sale_tag]
            df_ytrain['vote'] = np.where(abs(1 - df_ytrain['预估与实际比值']) < 0.5, 1, 0)
            df_ytrain['accuracy'] = df_ytrain['vote'].sum()/len(df_ytrain)
            df_train_summary = df_ytrain if not index else df_train_summary.append(df_ytrain)
            
            # 测试集真实数据及预测数据
            df_ytest = test_data.iloc[:, -1:]
            y_test_pred = model.predict(X_test)
            df_ytest['预估销量'] = y_test_pred * std[-1] + mean[-1]
            df_ytest['training_time'] = time_step
            df_ytest['预估与实际比值'] = df_ytest['预估销量'] / df_ytest[self.sale_tag]
            df_ytest['vote'] = np.where(abs(1 - df_ytest['预估与实际比值']) < 0.5, 1, 0)
            df_ytest['accuracy'] = df_ytest['vote'].sum() / len(df_ytest)
            df_test_summary = df_ytest if not index else df_test_summary.append(df_ytest)

            # 绘制散点图，分析预测结果与实际值的偏差
            tag1 = 'Train_' + mutil_tag + '_n' + str(time_step)
            tag2 = 'Test_' + mutil_tag + '_n' + str(time_step)
            self.correlation_analysis(self.sale_tag, "预估销量", df_ytrain, tag1, 'reg')
            self.correlation_analysis(self.sale_tag, "预估销量", df_ytest, tag2, 'reg')

        if effective_learning:
            if not mutil_NN:
                df_w2['参数'] = parameter
                df_w2.set_index('参数', inplace=True)
                df_w2.to_excel(os.path.join(self.time_tag, "各参数权重_%s.xlsx" % self.time_tag))

            # 合并训练和测试的summary
            df_train_summary["type"] = "training"
            df_test_summary["type"] = "test"
            df_summary = df_train_summary.append(df_test_summary)
            #df_summary.index.set_names("商圈", inplace=True)
            df_summary.sort_index(inplace=True)  # 按索引(即商圈名字)进行排序
            df_summary['商圈编号'] = np.arange(1, len(df_summary) + 1)  # data_visualization函数需商圈编号
            self.data_file = "summary_%s_%s.xlsx" % (mutil_tag, self.time_tag)
            df_summary.to_excel(os.path.join(self.time_tag, self.data_file))
            self.df = pd.read_excel(os.path.join(self.time_tag, self.data_file))
            self.boxplot("training_time", "预估与实际比值", "type")
        return effective_learning


def feature_extraction(feature_file):
    try:
        df = pd.read_excel(feature_file)
        parameter = list(df.iloc[:, 0])
    except:
        parameter = "火车站；网吧；便利店；中学；风景区；酒店；美食总数；小吃快餐；日本料理；"\
                    "火锅；烧烤；湖北菜；咖啡厅；湘菜；粤菜；面包甜点；西餐；外卖销量；人口；商圈面积"
        parameter = parameter.split('；')
        print("由于缺乏特征列表文件（%s），我们将使用一下特征列表\n" % feature_file, parameter)
    return parameter


# 集成学习
def ensemble_learning(work_dir, df, sales_file_proc, ocm_file, time_tag, sale_tag, sales_mapping_file, feature_file, province):
    if not os.path.exists("history"):
        os.mkdir("history")
    if not os.path.exists(time_tag):
        os.mkdir(time_tag)
    df = pd.read_excel(sales_mapping_file, index_col=0)
    parameter = feature_extraction(feature_file)
    df = df.loc[:, parameter + [sale_tag]]
    # 对缺失值进行填充
    df.fillna(0, inplace=True)
    step_list = [200]  # 50, 100, 200, 500, 1500]
    num = 0
    for i in range(100):
        time_tag2 = "%s_%s" % (time_tag, str(i + 1))  # time_tag2是为了分离每个弱学习器的学习结果
        bsp = BusinessSalesPrediction(work_dir, df, sales_file_proc, ocm_file, time_tag2, sale_tag, sales_mapping_file, province)
        effective_learning = bsp.neural_network_model(df, step_list)
        if effective_learning:
            num += 1

        if num >= 20:
            break

    num2 = 0
    for i in os.listdir('.'):
        if time_tag + '_' in i:  # 字符串中添加'_'是为了区分总文件夹time_tag
            df2 = pd.read_excel(os.path.join(i, "summary_MonoNN_%s.xlsx" % i))
            df_w = pd.read_excel(os.path.join(i, "各参数权重_%s.xlsx" % i), index_col=0)  # 将参数列设置为index
            df_w.drop('bias', inplace=True)
            df_w2 = df_w
            df_w.columns = ['权重']
            df2 = df2[df2['training_time'] == 200]  # 整体千箱运算目前只考虑200训练循环数
            df3 = df2 if not num2 else df3.append(df2, ignore_index=True)
            df_w1_all = df_w if not num2 else pd.concat([df_w1_all, df_w])
            df_w2_all = df_w2 if not num2 else pd.concat([df_w2_all, df_w2], axis=1)
            num2 += 1
            # 使用完之后，将文件夹移至history
            shutil.move(i, "history")

    df3.drop(['type'], axis=1, inplace=True)
    gb = df3.groupby("商圈")
    df4 = gb.median()
    df5 = gb.mean()
    df4.sort_values('预估与实际比值', ascending=False, inplace=True)  # 按销量变化率从大到小排序
    df_w1_all_median = df_w1_all.groupby('参数').median()
    df_w1_all_median.sort_values('权重', inplace=True)
    df_positive_factor = df_w1_all_median[::-1][:5]
    # df_positive_factor = df_positive_factor[df_positive_factor['权重'] > 0.1]
    df_positive_factor['正负性'] = '正因子'
    df_negative_factor = df_w1_all_median.head(5)
    # df_negative_factor = df_negative_factor[df_negative_factor['权重'] < -0.1]
    df_negative_factor['正负性'] = '负因子'
    df_factor = pd.concat([df_positive_factor, df_negative_factor])

    # 集成学习数据产出
    df3.to_excel(os.path.join(time_tag, '%s_all.xlsx' % time_tag), index=False)  # 所有弱学习器的产出数据
    df4.to_excel(os.path.join(time_tag, '%s_median.xlsx' % time_tag))  # 产出中位数
    df5.to_excel(os.path.join(time_tag, '%s_mean.xlsx' % time_tag))  # 产出平均值
    df_w1_all.to_excel(os.path.join(time_tag, "%s_weight_boxplot.xlsx" % time_tag))  # 纵向合并各弱学习器的参数权重，用于画boxplot
    df_w2_all.to_excel(os.path.join(time_tag, "%s_weight_heatmap.xlsx" % time_tag))  # 横向合并各弱学习器的参数权重，用于画热图
    df_w1_all_median.to_excel(os.path.join(time_tag, "%s_weight.xlsx" % time_tag))
    df_factor.to_excel(os.path.join(time_tag, '%s_正负因子.xlsx' % time_tag))  # 排名前五的正因子


def data_visualization(work_dir, time_tag, sale_tag, sales_file_proc, province):
    data_file1 = os.path.join(time_tag, "%s_weight_heatmap.xlsx" % time_tag)
    data_file2 = os.path.join(time_tag, "%s_weight_boxplot.xlsx" % time_tag)
    data_file3 = os.path.join(time_tag, "%s_all.xlsx" % time_tag)

    svl = SnsVisualization(work_dir, data_file1, time_tag)
    df1 = pd.read_excel(data_file1)
    df1.set_index("参数", inplace=True)
    # 绘制各地理因子的权重的热图
    svl.heatmap(df1, "弱学习器", "地理因子", "各地理因子的权重", annot=True, cmap="RdYlGn")
    # 绘制各地理因子的权重的箱线图
    pltz = PyplotZ()
    pltz.enable_chinese()
    df2 = pd.read_excel(data_file2)
    plt.figure(figsize=(14, 7))  # 输出图片的尺寸
    sns.set(style="ticks")
    plt.plot(np.arange(-1, 49), np.zeros(50), linestyle="--")  # 绘制0基线
    plt.xticks(rotation='45', fontproperties=zhfont1, fontsize=20)
    sns.boxplot(x='参数', y='权重', data=df2, color="#48D1CC")
    pltz.xlabel('参数')
    pltz.ylabel('权重')
    plt.savefig(os.path.join(time_tag, '%s_weight_boxplot.png' % time_tag), dpi=120, bbox_inches='tight')
    plt.close()
    # 使用箱线图显示各商圈的预估销量与真实销量的偏移水平
    df3 = pd.read_excel(data_file3)
    pltz = PyplotZ()
    pltz.enable_chinese()
    plt.figure(figsize=(16, 7))  # 输出图片的尺寸
    sns.set_style("whitegrid")
    plt.plot(np.arange(-1, 99), np.ones(100), linestyle="--")  # 绘制0基线
    plt.xticks(fontproperties=zhfont1)
    #  fliersize是异常值符号的大小
    sns.boxplot(x='商圈编号', y='预估与实际比值', data=df3, width=0.7, fliersize=1.5, color="#5CACEE", linewidth=0.8)
    pltz.xlabel('商圈编号')
    plt.yticks(np.arange(-2, 5, 1))
    pltz.ylabel('偏移值')
    plt.savefig(os.path.join(time_tag, '%s_商圈预估销量偏移.png' % time_tag), dpi=120, bbox_inches='tight')
    plt.close()
    # 箱线图-散点图解释商圈需要调整的原因
    adjustment_interpreter.interpreter(time_tag, sale_tag, sales_file_proc, province)


def main(work_dir, province, sales_file_ori, ocm_file, feature_file):
    sales_file_proc = sales_file_ori.split('.')[0] + '_proc.xlsx'
    if not os.path.exists(sales_file_proc):
         BusinessSalesPrediction(work_dir, 'df', sales_file_proc, ocm_file, 'time_tag', 'sale_tag',
                                 'sales_mapping_file', province).sum_mutil_dealer(sales_file_ori)

    sale_data = pd.read_excel(os.path.join(work_dir, sales_file_proc), index_col=0)
    time_tag = time.strftime("%Y-%m-%d-%Hh%Mm", time.localtime())
    for sale_tag in ['整体平均千箱', '高价平均千箱']:
        tag = "%s_%s_" % (province, sale_tag) + time_tag
        df = pd.DataFrame({sale_tag: sale_data[sale_tag]})
        sales_mapping_file = "%s商圈OCM_%s.xlsx" % (province, sale_tag)  # sales_mapping_file为中间过度文件，经常被覆盖
        # if not os.path.exists(sales_mapping_file):
        bsp = BusinessSalesPrediction(work_dir, df, sales_file_proc, ocm_file, tag, sale_tag, sales_mapping_file, province)
        bsp.ocm_pretreatment()
        ensemble_learning(work_dir, df, sales_file_proc, ocm_file, tag, sale_tag, sales_mapping_file, feature_file, province)
        data_visualization(work_dir, tag, sale_tag, sales_file_proc, province)
        # 暂时关闭商圈等级分析：AnalyzeGrading.analyze_grading(tag, '%s_综合.xlsx' % tag, '客户形态对应的商圈等级', '预估销量', '整体预估销量', province)


if __name__ == "__main__":
    work_dir = "."
    sales_file_ori = "湖北业绩信息1218.xlsx"
    ocm_file = "湖北_OCM参数.xlsx"#"湖北各区县OCM_20171219.xlsx"
    main(work_dir, province, sales_file_ori, sales_file_proc, ocm_file)




# ---------------------------------------------------------------------------------------------------------------------#
# AUTHOR: Jasonai, jinxiang_zhu@parramountain.com                                                                        #
# ORGANIZATION: PMI                                                                                                    #
# VERSION: 1.0                                                                                                         #
# CREATED: 3rd Mar 2018                                                                                                #
# ---------------------------------------------------------------------------------------------------------------------#

import os
import pandas as pd

#import seaborn as sns
#import matplotlib.pyplot as plt
#from sklearn.preprocessing import StandardScaler

"""Tnseq command line interface """

# =============== <Realization of shell scripts using click package> #==============#
import click

VERSION = '0.12'
CONTEXT_SETTINGS = dict(help_option_names=['-h', '--help'])


@click.group(context_settings=CONTEXT_SETTINGS)
@click.version_option(version=VERSION)
def main():
    pass


@main.command(help='根据经纬度批量提取宜出行人流数据')
@click.option('--file_path1', '-f1', default='', help='存放店铺/点位经纬度数据的文件，格式为xlsx或csv')
@click.option('--file_path2', '-f2', default='', help="存放宜出行数据的文件，格式为xlsx或csv，且表头列名需包含['lng', 'lat', 'count']")
@click.option('--lnglat', '-ll',multiple=True, default=['经度','纬度'], help='输入f1文件存放存放经度、纬度的列名，默认为“-l 经度 -l 纬度”')
@click.option('--meter', '-m', default='500', help='指定获取宜出行点位的范围(半径)，输入格式为正整数，默认为500(米)')
@click.option('--core_num', '-c', default='max', help='指定程序并发运行的线程数，输入格式为正整数，默认值为本地计算机最大的线程数')
@click.option('--file_ouput', '-o', default='', help='输入输出文件名，默认为f1,f2文件名的合并，输出格式为xlsx格式')
def yichuxing(**kwargs):
    from pmipy import getPeopleFlow
    pointFile = kwargs['file_path1']
    ycxFile = kwargs['file_path2']
    lnglat = kwargs['lnglat']
    meter = kwargs['meter']
    core_num = kwargs['core_num']
    outputFile = kwargs['file_ouput']
    getPeopleFlow.getPeopleFlow(pointFile, ycxFile, lnglat, meter, core_num, outputFile)


@main.command(help='地理编码或逆编码/坐标转换')
@click.option('--file_path', '-f', prompt='请输入文件路径', help='存放地理位置信息的文件，格式为xlsx或csv')
@click.option('--address', '-a',multiple=True, default=['地址'], help='输入描述性地址所在列的列名，多列请输入“-a column1 -a column2 ...”，\
比如需要文件中“省”、“市”和“具体地址”这三列的内容合并进行搜索经纬度度，则填写“-a 省 -a 市 -a 具体地址”，默认值为“-a 地址”')
@click.option('--tranform', '-tf', is_flag=True, help='经纬度坐标转换开关，只需要坐标转换时填写“-tf”即可！')
@click.option('--lnglat', '-l',multiple=True, default=['经度','纬度'], help='输入f文件存放经度、纬度的列名，默认为“-l 经度 -l 纬度”')
@click.option('--initial_coordinate', '-ic', default='gaode', help='填写原始坐标的格式标准，如为高德火星坐标系，则填写“gaode”;\
如为百度坐标系填写“baidu”;国际大地坐标系填写“global”，默认为gaode')
@click.option('--target_coordinate', '-tc', default='gaode', help='填写输出坐标的格式标准，如为高德火星坐标系，则填写“gaode”;\
如为百度坐标系填写“baidu”;国际大地坐标系填写“global”，默认为gaode')
@click.option('--thread', '-t', default=50, help='指定程序并发运行的线程数，输入格式为正整数，默认值为50')
@click.option('--file_ouput', '-o', default='', help='输入输出文件名，默认文件的前缀为f的文件名')
def coordinate(**kwargs):
    from pmipy import getTargetCoordinate
    file_path = kwargs['file_path']
    address = kwargs['address']
    tranform = kwargs['tranform']
    lnglat = kwargs['lnglat']
    initial_coordinate = kwargs['initial_coordinate']
    target_coordinate = kwargs['target_coordinate']
    thread = kwargs['thread']
    file_ouput = kwargs['file_ouput']
    if tranform:
        getTargetCoordinate.coordinateTransform(file_path, file_ouput, lnglat, initial_coordinate, target_coordinate)
    else:
        getTargetCoordinate.getTargetCoordinate(file_path, address, thread, target_coordinate, file_ouput)


@main.command(help='特征工程器<特征处理/特征选择>')
@click.option('--feature_file', '-f1', prompt='请输入特征文件路径', help='存放特征数据表的文件，格式为xlsx或csv')
@click.option('--label_file', '-f2', default='', help='存放标签数据表的文件，如标签在上述特征文件中，则可以不用填写，格式为xlsx或csv')
@click.option('--colf', '-cf',default='编号', help='colF和colL为特征和标签数据的索引列名（即vlookup列），默认为“编号”')
@click.option('--coll', '-cl',default='编号', help='colF和colL为特征和标签数据的索引列名（即vlookup列），默认为“编号”')
@click.option('--label', '-l',default='标签', help='机器学习的标签列名，默认为“标签”')
@click.option('--border', '-b',default='mean', help='针对回归项目的数值型标签：此选项可指定二分类化的方法，默认为“mean”,\
其他备选可以是“median”、“kmeans”等')
@click.option('--group_num', '-gn',default=4, help='针对WOE编码：分组时的组别数量，默认为4组')
@click.option('--comp_num', '-cn',default=4, help='针对特征分析：特征复杂度，默认为4')
@click.option('--group_meth', '-gm',default=1, help="针对WOE编码：分组的方法。1:'等距法,2:'自身类别法'。'默认为“等距法”")
@click.option('--file_ouput', '-o', default='', help='输入输出文件名，默认文件的前缀为f1的文件名')
def featreng(**kwargs):
    from pmipy import featureEngineer
    featureFile = kwargs['feature_file']
    labelFile = kwargs['label_file']
    colF = kwargs['colf']
    colL = kwargs['coll']
    label = kwargs['label']
    border = kwargs['border']
    groupNum = kwargs['group_num']
    compNum = kwargs['comp_num']
    groupMeth = kwargs['group_meth']
    file_ouput = kwargs['file_ouput']
    featureEngineer.featureEngineering(featureFile, labelFile, colF, colL, label, border, groupNum, compNum, groupMeth, file_ouput)


@main.command(help='OCM参数可视化分析')
@click.option('--data_path', '-f', default='湖北人口_ocm1.5.csv', help="输入待分析的OCM文件路径，默认为“湖北人口_ocm1.5.csv”")
@click.option('--tag', '-t', default='湖北人口', help='用于存放输出文件的文件夹，默认为“湖北人口”')
def OCM_Visual(**kwargs):
    from pmipy import OCMVisualization
    data_path = kwargs['data_path']
    tag = kwargs['tag']
    path =os.path.dirname(OCMVisualization.__file__)  # OCMFeatureExtract模块所在的路径
    OCMVisualization.run_Rscript(data_path, tag, path)


@main.command(help='提取OCM参数')
@click.option('--data_path', '-f', default='湖北需求预估使用的参数.xlsx', help='输入待提取的参数信息文件路径，默认为“湖北需求预估使用的参数.xlsx”')
@click.option('--province', '-p', default='湖北', help='选择提取的省份，默认为湖北')
def OCM_extract(**kwargs):
    from pmipy import OCMFeatureExtract
    feature_file = kwargs['data_path']
    province = kwargs['province']
    df = pd.read_excel(feature_file, index_col=[0])  # 将'省'列设为索引
    OCMFeatureExtract.merging_data(df, province)


@main.command(help='分析销售数据或OCM参数')
@click.option('--data_path', '-f', default='福州上海子公司业绩表.xlsx', help='分析数据的文件路径')
@click.option('--index', '-i', default='餐厅名称', help='选择某列作为数据框的索引')
@click.option('--value_col', '-vc', default=5, help='选择用于数据标准化处理的开始列数')
def feature_analysis(**kwargs):
    data_path = kwargs['data_path']
    index = kwargs['index']
    # value_col = kwargs['value_col']
    df = pd.read_excel(data_path, index_col=index)
    print(df.head())


@main.command(help='BIT自动化程序')
@click.option('--file_path1', '-f1', default='ts2.xlsx', help='待写入的excel文件路径，默认为“ts2.xlsx”')
@click.option('--file_path2', '-f2', default='MR002-170802Y-德克士Brand Image Tracking（W45-W48）（11月）数据v2.xlsx', 
help='问卷信息文件（已初步清洗的excel文件）路径，默认为“MR002-170802Y-德克士Brand Image Tracking（W45-W48）（11月）数据v2.xlsx”')
@click.option('--file_path3', '-f3', default='qa-Olivia.xlsx', help='问卷题目编号信息文件路径，如未来题目有变动，需更新此文件信息，默认为“qa-Olivia.xlsx”')
@click.option('--file_path4', '-f4', default='新配额分布.xlsx', help='城市配额/权重文件路径，默认为“新配额分布.xlsx”')
@click.option('--sheet', '-s', default='sheet2', help='选择待填写的sheet。sheet2:<2.外食情况>;sheet3:<3.品牌与广告知名度>;sheet4:<4.品牌购买与食用>;\
sheet5:<5.西式快餐U&A >;sheet6:<6.重要指标汇总>;sheet7:<7.样本资料>，如填写“all”，则表示写入所有sheet。默认为“sheet2”')
@click.option('--period_num', '-p', default=4, help='输入阶段数，默认为4')
def BIT(**kwargs):
    from pmipy import BIT
    filepath1 = kwargs['file_path1']
    filepath2 = kwargs['file_path2']
    filepath3 = kwargs['file_path3']
    filepath4 = kwargs['file_path4']
    sheet = kwargs['sheet']
    period_num = kwargs['period_num']
    BIT.main(sheet, filepath1, filepath2, filepath3, filepath4, period_num)


@main.command(help='需求预估（商圈分级）')
@click.option('--work_dir', '-wd', default='.', help='输入数据的存放目录，默认为当前工作目录')
@click.option('--sales_file_ori', '-sfo', default='湖北业绩信息1218.xlsx', help='销售数据的初始文件路径，默认为“湖北业绩信息1218.xlsx”')
# @click.option('--sales_file_proc', '-sfp', default='', help='处理过的销售数据路径，如存在则程序会无视sales_file_ori文件，默认不存在')
@click.option('--ocm_file', '-of', default='湖北_OCM参数.xlsx', help='OCM参数的文件路径，默认为“湖北_OCM参数.xlsx”')
@click.option('--feature_file', '-ff', default='', help='OCM参数的名单路径，默认不填')
@click.option('--province', '-p', default='湖北', help='选择预测的省份，默认为湖北')
def demand_predict(**kwargs):
    from pmipy import demand_predict
    work_dir = kwargs['work_dir']
    sales_file_ori = kwargs['sales_file_ori']
    # sales_file_proc = kwargs['sales_file_proc']
    ocm_file = kwargs['ocm_file']
    feature_file = kwargs['feature_file']
    province = kwargs['province']
    demand_predict.main(work_dir, province, sales_file_ori, ocm_file, feature_file)


# ================================== <Running main function> ================================== #
if __name__ == '__main__':
    main()





import os
import pandas as pd


def cal_shipment(df):
    # 纳入计算的品类名单
    category_list = ['容器面', '高价袋','中价面',"平低价","干脆面",
         '容器面.1', '高价袋.1','中价面.1',"平低价.1","干脆面.1",
         '容器面.2', '高价袋.2','中价面.2',"平低价.2","干脆面.2"]
    # 提取上述三部分信息
    df = df[['区/县', '具体描述'] + category_list]
    # 处理区域、商圈信息的合并单元格
    df[['区/县', '具体描述']] = df[['区/县', '具体描述']].fillna(method="pad")
    # 这一步很重要。各品类出货量数字合并单元格需填0处理
    df[category_list] = df[category_list].fillna(0)
    # 获取一区多商圈的样本
    split_df = df.loc[:, ['具体描述','区/县']]
    split_df.drop_duplicates(['具体描述','区/县'], inplace=True)
    split_df1 = split_df.groupby(['区/县']).count()
    split_df2 = split_df1[split_df1['具体描述'] > 1].reset_index()
    # 计算区县分销量
    split_df3 = split_df2.loc[:,['区/县']].merge(df.loc[:, ['区/县','具体描述','容器面.2', '高价袋.2','中价面.2',
                             "平低价.2","干脆面.2"]],on='区/县', how='left')
    split_df3['区县分销量'] = split_df3.loc[:, ['容器面.2', '高价袋.2','中价面.2',"平低价.2","干脆面.2"]].sum(axis=1)
    # 如存在一商圈多经销部，则合并商圈销量
    split_df4 = split_df3.groupby(['区/县', '具体描述'], as_index=False).sum()[['区/县', '具体描述', '区县分销量']]
    split_df4.set_index('区/县', inplace=True)
    split_df4['区县总销量'] = split_df4.groupby('区/县').sum()
    #split_df5 = split_df5.reset_index(level='区/县')
    allocation_df = split_df.merge(split_df4.reset_index(), on=['具体描述','区/县'], how='outer')
    allocation_df.columns = ['具体描述','区县', '区县分销量', '区县总销量']
    allocation_df.set_index(['具体描述','区县'], inplace=True)
    # 营业部放第一列
    final_df=df.groupby(['具体描述']).sum()
    return final_df, allocation_df

def sum_TT_MT(path, TT_file,MT_file, province):
    # 读取TT数据表, TT数据必须放在活动sheet
    df1 = pd.read_excel(TT_file,skiprows=2)
    # 读取MT数据表
    try:
        df2 = pd.read_excel(MT_file,skiprows=2, sheetname='MT(不含后面3张sheet标红系统)')
    except:
        # MT出货数据存放的sheet在活动sheet中
        df2 = pd.read_excel(MT_file,skiprows=2)
    
    # 此接口可灵活增减额外信息，比如经销商信息
    add_info = ['具体描述','省','市','区/县','营业部',"现有经销商/物流公司名称","客户形态","现有经销商/物流公司代码"]
    df_info = df1[add_info].fillna(method="pad")
    df_info.set_index('具体描述', inplace=True)
    TT_df, TT_allocation_df = cal_shipment(df1)
    MT_df, MT_allocation_df = cal_shipment(df2)
    TTMT_df = TT_df.copy()
    TTMT_allocation_df = TT_allocation_df.copy()
    # merged_df=pd.merge(TT_df,MT_df,on=["具体描述"])
    # 营业部在第一列
    for spec in TT_df:
        TTMT_df[spec] = TTMT_df[spec] + MT_df[spec]
    
    # 计算各区县分配系数
    for row in TT_allocation_df.columns:
        TTMT_allocation_df[row] = TTMT_allocation_df[row] + MT_allocation_df[row]
    TTMT_allocation_df['区县分配系数'] = TTMT_allocation_df['区县分销量'] / TTMT_allocation_df['区县总销量']
    TTMT_allocation_df['区县分配系数'] = TTMT_allocation_df['区县分配系数'].fillna(1)
    # 输出KSF各商圈TT&MT出货
    writer=pd.ExcelWriter(os.path.join(path, "{0}KSF各商圈TT&MT出货.xlsx".format(province)))
    TTMT_df.to_excel(writer,sheet_name="TT&MT", merge_cells=False)
    TT_df.to_excel(writer,sheet_name="TT", merge_cells=False)
    MT_df.to_excel(writer,sheet_name="MT", merge_cells=False)
    writer.save()
    # KSF各商圈区县分配系数
    writer2=pd.ExcelWriter(os.path.join(path, "{0}KSF各商圈区县分配系数.xlsx".format(province)))
    TTMT_allocation_df.to_excel(writer2,sheet_name="TT&MT", merge_cells=False)
    TT_allocation_df.to_excel(writer2,sheet_name="TT", merge_cells=False)
    MT_allocation_df.to_excel(writer2,sheet_name="MT", merge_cells=False)
    writer2.save()
    # 输出商圈经销商等额外信息
    df_info.to_excel('%s商圈信息.xlsx' % province)


if __name__ == '__main__':
    province = '湖北省'
    TT_file = '湖北业绩数据（TT+MT）.xlsx'
    MT_file = '湖北业绩数据（TT+MT）.xlsx'
    path = os.path.dirname(TT_file)
    sum_TT_MT(path, TT_file,MT_file, province)
    



# -*- coding: utf-8 -*-
"""
Created on Wed Apr  4 16:27:28 2018

@author: jasonai
"""
import numpy as np
import pandas as pd



# 考虑的通路点数
channel = ["中学","加油加气站","小学","成人教育","火车站","网吧","超市","酒店","长途汽车站","高等院校"]
def grading_standard(df, province='湖北省'):
    gb = df.groupby('原先等级')
    df_q1 = gb.quantile(q=0.25)
    df_q1['统计'] = '下四分位'
    df_q2 = gb.quantile(q=0.5)
    df_q2['统计'] = '中位数'
    df_q3 = gb.quantile(q=0.75)
    df_q3['统计'] = '上四分位'
    df2 = pd.concat([df_q1, df_q2, df_q3])
    df3 = df2.sort_index()
    df3.to_excel('各等级信息.xlsx')
    return df3


df = pd.read_excel('湖北商圈分级调整建议2.xlsx')
# df2 = grading_standard(df)

# df['建议等级'] = df['原先等级']
# 其他
df['形态1'] = '其他'
# 定义核心城区
df.ix[(df['总市场量(千箱/月)'] >= 180) & (df['通路点数']>2500) & ((df['高价面']>15) | (df['人口']>500)), '形态1'] = '核心城区'
df.ix[(df['形态1'] == '核心城区') & (df['人口']>200), '商圈形态'] = '核心城区A'
df.ix[(df['形态1'] == '核心城区') & (df['人口']<=200), '商圈形态'] = '核心城区B'
# 精耕城区
df.ix[(df['形态1'] != '核心城区') & (df['总市场量(千箱/月)'] >= 80) & (df['高价面']>11), '形态1'] = '精耕城区'
df.ix[df['形态1'] == '精耕城区', '商圈形态'] = '精耕城区'
# 准精耕城区
df.ix[(df['形态1'] == '其他') & (df['总市场量(千箱/月)'] >= 50) & (df['高价面']>6) & (df['人口']>45), '形态1'] = '准精耕城区'
df.ix[(df['形态1'] == '准精耕城区') & ((df['人口']>80) | (df['总市场量(千箱/月)'] >= 70)), '商圈形态'] = '准精耕城区A'
df.ix[(df['形态1'] == '准精耕城区') & (df['形态1'] !='准精耕城区A'), '商圈形态'] = '准精耕城区B'

# 其他-外阜
df.ix[(df['大商圈'] == '外埠片区') & (df['形态1'] == '其他') & (df['总市场量(千箱/月)'] > 40), '商圈形态'] = '外埠A'
df.ix[(df['大商圈'] == '外埠片区') & (df['形态1'] == '其他') & (df['总市场量(千箱/月)'] > 30) & (df['总市场量(千箱/月)'] <= 40), '商圈形态'] = '外埠B'
df.ix[(df['大商圈'] == '外埠片区') & (df['形态1'] == '其他') & (df['总市场量(千箱/月)'] > 15) & (df['总市场量(千箱/月)'] <= 30), '商圈形态'] = '外埠C'
df.ix[(df['大商圈'] == '外埠片区') & (df['形态1'] == '其他') & (df['总市场量(千箱/月)'] < 15), '商圈形态'] = '外埠C'

# 其他
df.ix[(df['大商圈'] != '外埠片区') & (df['形态1'] == '其他') & (df['总市场量(千箱/月)'] > 40), '商圈形态'] = '核心郊区县A'
df.ix[(df['大商圈'] != '外埠片区') & (df['形态1'] == '其他') & (df['总市场量(千箱/月)'] > 20) & (df['总市场量(千箱/月)'] <= 40), '商圈形态'] = '核心郊区县B'
df.ix[(df['大商圈'] != '外埠片区') & (df['形态1'] == '其他') & (df['总市场量(千箱/月)'] < 20), '商圈形态'] = '核心郊区县C'

df2 = pd.read_excel('湖北商圈分级调整建议2.xlsx', sheetname='等级信息对应表')
df_merge = df.merge(df2, on='商圈形态', how='left')
df_merge = df_merge.rename(columns={"商圈等级": "建议等级"})
# 升降级
df_merge['调整建议'] = '维持'
df_merge.ix[df_merge['原先等级'] > df_merge['建议等级'], '调整建议'] = '建议升级'
df_merge.ix[df_merge['原先等级'] < df_merge['建议等级'], '调整建议'] = '建议降级'
print('建议升级：',len(df_merge[df_merge['调整建议'] == '建议升级']))
print('建议降级：', len(df_merge[df_merge['调整建议'] == '建议降级']))
df_merge.to_excel('ts.xlsx', index=False)

#get_channel('湖北省各商圈总市场量（千箱）')
# get_channel('湖北省各商圈人口')
#get_channel('湖北省各商圈人均包')
"""
df1 = pd.read_excel('湖北省各商圈总市场量（千箱）_含等级信息.xlsx')
df1.drop('湖北省', inplace=True)
df2 = pd.read_excel('湖北省各商圈通路点数.xlsx', index_col='具体描述',usecols=['具体描述','通路点数'])
df3 = pd.read_excel('湖北省各商圈人均包.xlsx',index_col='具体描述',usecols=['具体描述','高价面'])
df = pd.concat([df1,df2,df3], axis=1)
df = pd.concat([df,df3], axis=1)
"""






df['状态'] = '维持不变'
df.ix[df['等级'] > df['建议等级'], '状态'] = '建议升级'
df.ix[df['等级'] < df['建议等级'], '状态'] = '建议降级'



df5 = pd.read_excel('湖北省各商圈地理因子数据.xlsx', index_col='具体描述')
df6 = df5[channel]
df6['通路点数'] = df6.sum(axis=1)
df6.to_excel('湖北通路点数.xlsx')


# -*- coding: utf-8 -*-
"""
Created on Sat Apr 14 11:23:05 2018

@author: jasonai
"""

import os
import numpy as np
import pandas as pd
from pmipy import log
from pmipy import pmi

logger = log.createLogger(__name__)
# 输出不同级别的log


def readXlsCsv(filePath):
    [path, file] = os.path.split(filePath)
    [fileName, fileSuffix] = os.path.splitext(file)
    if fileSuffix == '.xlsx' or fileSuffix == '.xls':
        df = pd.read_excel(filePath)
    elif fileSuffix == '.csv':
        try:
            df = pd.read_csv(filePath, engine='python')
        except pd.errors.ParserError: # csv为tab键分割的情况
            df = pd.read_csv(filePath, sep='\t', engine='python')
    else:
        logger.error("Please input csv or xlsx file!")
        raise SystemExit(1) 
        
    return df, path, fileName


# 对特征、标签进行预处理
class pretreatment(object):
    def __init__(self, featureFile, labelFile):
        self.featureFile = featureFile
        self.labelFile = labelFile
        # 输出文件的目录为存放特征文件的目录，文件名为特征文件名的拓展
        self.df_feature, self.path, self.fileName = readXlsCsv(featureFile)


    # 合并、简单处理特征文件和标签文件,并对标签进行二分类；colF和colL为特征和标签数据的索引列名（vlookup列）
    def featureLabel(self, colF='编号', colL='编号', label='label', border='mean'): 

        if not self.labelFile: # 判断是否指定了labelData文件路径，默认labelFile为''
            if label not in self.df_feature.columns: 
                logger.error("请指定存放标签数据表文件名/文件路径! <命令行提示：-f2 pathOfLabelFile >")
                raise SystemExit(1) 
            else:
                df_merge = self.df_feature
        else:
            df_label = readXlsCsv(self.labelFile)[0]
            df_merge = self.df_feature.merge(df_label[[colL, label]], left_on=colF, right_on=colL, how='inner')
    
        # 降序,确保1分类为1
        df_merge.sort_values(label, inplace=True)
        # 获取标签复杂度
        labelNum = len(df_merge[label].drop_duplicates())
        
        # 数值型标签处理,进行二分类
        try:
            # 如果发现数字型series只有两个值，我们暂时当中分类问题
            if labelNum  == 2:
                raise TypeError
            
            logger.info("使用%s方法进行二分类数值型标签 ..." % border)
            if border == 'mean':
                df_merge['cls'] = np.where(df_merge[label] <= df_merge[label].mean(), 0, 1)
            elif border == 'median':
                df_merge['cls'] = np.where(df_merge[label] <= df_merge[label].median(), 0, 1)
            elif border == 'kmeans':
                from sklearn.cluster import KMeans
                estimator=KMeans(n_clusters=2)
                res=estimator.fit_predict(pd.concat([df_merge[label],df_merge[label]],axis=1))
                df_merge['cls'] = [abs(1-x) for x in res]
                # logger.exception('导入KMeans模块失败！')
            elif border == 'self':
                df_merge['cls'] = df_merge[self.labelFile]
            else:
                logger.error("%s方法获得二分类的分界值尚未开发，如有需求请联系开发人员<pmi-jasonai>!" % border)
                raise SystemExit(1)
                
        except TypeError:
            if labelNum  > 2:
                logger.error("所得标签的数据类型为非数字型(dtype:object)，因此只能视为分类问题，但WOE特征选择算法暂时只考虑二分类问题！")
                raise SystemExit(1)
            elif labelNum == 2:
                df_merge['cls'] = np.where(df_merge[label] == df_merge[label].drop_duplicates()[0], 0, 1)
            else: # 如果标签只有1种值
                logger.warning("所获取的标签只有一种值，请仔细检查标签是否存在问题！")
                SystemExit(1)
    
        return df_merge

    # 对特征进行缺失值处理  
    def missingValue(self, percent):
        series_list = []
        for col in self.df_feature.columns:
            series = self.df_feature[col]
            if series[series.isnull()].count() / series.count() > percent:
                logger.info("特征“{}”的缺失值占比超过{:.1f},舍弃此特征".format(series.name, percent))
            else:
                # 暂时不做其他处理，未来有时间可能会在此加稀疏度之类的分析
                # series.fillna(0)
                series_list.append(series)
        self.df_feature = pd.concat(series_list, axis=1)
        return self.df_feature
        
    # 分析标签的复杂度,舍弃复杂度小于num的标签
    def analyzeComplex(self, num=None):
        # 如果没有提供复杂度要求
        if not num:
            num = 4
        series_list = []
        for col in self.df_feature.columns:
            series = self.df_feature[col]
            complexNum = len(series.drop_duplicates())
            if complexNum < num:
                logger.info("特征'{}'的复杂度为{}，小于指定的{},舍弃此特征".format(series.name, complexNum, num))
            else:
                series_list.append(series)
        self.df_feature = pd.concat(series_list, axis=1)
        return self.df_feature


class featureWOE(object):
    def __init__(self, df):
        self.df = df

    # 对特征进行分组
    def getWOEandIV(self, groupNum=4, groupMeth=1):
        IV_list = []
        if groupMeth == 1:
            logger.info("使用等距法进行分{}组 ...".format(groupNum))
            sample_size = len(self.df)
            for feature in self.df.columns[:-2]: # 最后两列,'label'和'cls'列为标签相关列
                df_uniF = self.df.sort_values(feature)
                df_uniF['group'] = np.arange(sample_size) // (sample_size / groupNum)
                WOE, IV = self.calWOEandIV(df_uniF)
                IV_list.append([feature, WOE, IV])
            
        elif groupMeth == 2:
            logger.info("使用自身分类法进行分组 ...")
            for feature in self.df.columns[:-2]:
                 df_uniF = self.df.sort_values(feature)
                 WOE, IV = self.calWOEandIV(df_uniF, var=feature)
                 IV_list.append([feature, WOE, IV])
        else:
            pass

        df_ouput = pd.DataFrame(IV_list, columns=['特征', 'WOE', 'IV'])
        
        return df_ouput
        

    def calWOEandIV(self, df_uniF, var='group', target='cls'):
        eps = 0.000001 # 以避免除以0
        gbi = pd.crosstab(df_uniF[var], df_uniF[target]) + eps
        gb = df_uniF[target].value_counts() + eps
        gbri = gbi / gb
        gbri['woe'] = np.log(gbri[1] / gbri[0])
        gbri['iv'] = (gbri[1] - gbri[0]) * gbri['woe']
        
        return gbri['woe'].to_dict(), gbri['iv'].sum()


@pmi.execInfo()
def featureEngineering(featureFile, labelFile, colF, colL, label, border, groupNum, compNum, groupMeth='',file_ouput=''):
    # 初始化pretreatment类
    pret = pretreatment(featureFile, labelFile)
    # 先进行特征预处理
    # 缺失值处理：去除缺失值超过60%的特征
    pret.missingValue(percent=0.6)
    # 去除复杂度低于样本数10分之一的特征
    pret.analyzeComplex(compNum)
    # 处理并合并标签
    df_fela = pret.featureLabel(colF, colL, label, border)
    # 初始化featureWOE类
    fw = featureWOE(df_fela)
    dfWOE = fw.getWOEandIV(groupNum, groupMeth)
    dfWOE.to_excel(os.path.join(pret.path, pret.fileName+'_woe.xlsx'), index=False)


if __name__ == '__main__':
    featureFile = ''
    labelFile = ''
    colF, colL, label = '', '', ''
    featureEngineering(featureFile, labelFile, colF, colL, label)



# Analysis of Business Circle 's Grading Rationality Based on Real Sales

import os
import pandas as pd
import seaborn as sns
import matplotlib as mpl
import matplotlib.pyplot as plt
from pyplotz.pyplotz import PyplotZ


# 解决matplotlib和seaborn绘图中文乱码的问题（坐标轴元素乱码）
zhfont1 = mpl.font_manager.FontProperties(fname='C:\Windows\Fonts\simkai.ttf')


def boxplot(df, x, y, data_path, hue=None):
    pltz = PyplotZ()
    pltz.enable_chinese()  # 解决坐标标签乱码的问题
    sns.set_style("white")  # sns.set(style="ticks")
    plt.xticks(fontproperties=zhfont1)
    plt.yticks(fontproperties=zhfont1)
    sns.boxplot(x=x, y=y, hue=hue, data=df, palette="PRGn")
    pltz.xlabel(x)
    pltz.ylabel(y)
    sns.swarmplot(x=x, y=y, hue=hue, data=df)
    plt.savefig("%s/%s_swarmplot" % (data_path, y))
    plt.close()


def analyze_grading(data_path, data_file, grade, sale, tag, province):
    # 读取各商圈销量数据
    df = pd.read_excel(os.path.join(data_path, data_file), index_col=0)
    df.sort_values(by=grade, inplace=True)
    boxplot(df, grade, sale, data_path)

    # 根据上图（箱线图）个等级的销量分布，
    # 仅考虑整体平均千箱数据，通过商圈等级进行分组，最终通过四分位和最小值找出不合理的分级商圈
    df2 = df.loc[:, [grade, sale]]
    gb = df2.groupby(grade)

    # 获取各等级的下四分位Q1销量数据
    df_q1 = gb.quantile(q=0.25)

    # 获取各等级的中位Q2销量数据
    df_q2 = gb.quantile(q=0.5)

    # 获取各等级的上四分位Q3销量数据
    df_q3 = gb.quantile(q=0.75)

    """
    分析商圈分级合理性的方法1：
    某等级的商圈销量超过上一级的销量中位值即视为分级过低，
    而当销量小于下一级的销量中位值即视为分级过高。

    分析商圈分级合理性的方法2：
    从箱线图显示的6个等级销量分布结果中，我们可以看出1级商圈和2级商圈的销量拉的最开，而2级以下的等级，每个级别都存在较多的销量重叠。
    根据分布结果，我们拟定：
    1级商圈中，销量低于2级上四分位的商圈标记为不合理分级的商圈；
    2级商圈中，销量低于3级下四分位的商圈标记为不合理分级的商圈；
    3级商圈中，销量低于4级下四分位的商圈标记为不合理分级的商圈；
    4级商圈中，销量低于5级下四分位的商圈标记为不合理分级的商圈；
    5级商圈中，销量低于6级下四分位的商圈标记为不合理分级的商圈；
    2级商圈中，销量超过1级下四分位的商圈标记为不合理分级的商圈；
    3级商圈中，销量超过2级中位数的商圈标记为不合理分级的商圈；
    4级商圈中，销量超过3级上四分位的商圈标记为不合理分级的商圈；
    5级商圈中，销量超过4级上四分位的商圈标记为不合理分级的商圈；
    6级商圈中，销量超过5级上四分位的商圈标记为不合理分级的商圈；
    """
    evaluate_tag = []
    for grade, sale in df2.values:
        if grade == '1级':
             if sale < df_q2.loc['2级'].values:
                 evaluate_tag.append('分级过高')
             else:
                 evaluate_tag.append('分级合理')
        elif grade == '6级':
                 if sale > df_q2.loc['5级'].values:
                    evaluate_tag.append('分级过低')
                 else:
                    evaluate_tag.append('分级合理')
        else:
            last_grade = str(int(grade[0]) - 1) + '级'
            next_grade = str(int(grade[0]) + 1) + '级'
            if sale < df_q2.loc[next_grade].values:
                evaluate_tag.append('分级过高')
            elif sale > df_q2.loc[last_grade].values:
                evaluate_tag.append('分级过低')
            else:
                evaluate_tag.append('分级合理')
    df2['%s分析分级合理性' % tag] = evaluate_tag
    df2.to_excel('%s/基于%s分析%s商圈分级合理性.xlsx' % (data_path, tag, province))

"""
    方法2：
    for grade, sale in df2.values:
        if grade == '1级':
            if sale < df_q3.loc['2级'].values:
                evaluate_tag.append('分级过高')
            else:
                evaluate_tag.append('分级合理')
        elif grade == '2级':
            if sale < df_q1.loc['3级'].values:
                evaluate_tag.append('分级过高')
            elif sale > df_q3.loc['1级'].values:
                evaluate_tag.append('分级过低')
            else:
                evaluate_tag.append('分级合理')
        elif grade == '3级':
            if sale < df_q1.loc['4级'].values:
                evaluate_tag.append('分级过高')
            elif sale > df_q2.loc['2级'].values:
                evaluate_tag.append('分级过低')
            else:
                evaluate_tag.append('分级合理')
        elif grade == '4级':
            if sale < df_q1.loc['5级'].values:
                evaluate_tag.append('分级过高')
            elif sale > df_q3.loc['3级'].values:
                evaluate_tag.append('分级过低')
            else:
                evaluate_tag.append('分级合理')
        elif grade == '5级':
            if sale < df_q1.loc['6级'].values:
                evaluate_tag.append('分级过高')
            elif sale > df_q3.loc['4级'].values:
                evaluate_tag.append('分级过低')
            else:
                evaluate_tag.append('分级合理')
        else:  # 6级
            if sale > df_q3.loc['5级'].values:
                evaluate_tag.append('分级过低')
            else:
                evaluate_tag.append('分级合理')
"""


if __name__ == '__main__':
    data_path = '整体平均千箱_2017-12-04-15h00m'
    data_file = '整体平均千箱_2017-12-04-15h00m_综合.xlsx'
    grade = '客户形态对应的商圈等级'
    sale = '预估销量'
    tag = '整体预估销量'
    analyze_grading(data_path, data_file, grade, sale, tag)
    analyze_grading('.', '湖北各商圈销量.xlsx', grade, '整体平均千箱', '整体平均销量')



# -*- coding: utf-8 -*-
"""
Created on Thu Apr  5 19:18:05 2018

@author: jasonai
"""

import os
import yaml
import pandas as pd
from pymongo import MongoClient

# 连接OCM1.6
def conn_ocm16(LAN=True):
    IP_LAN = '192.168.0.21:27017'
    IP_PN = 'www.parramountain.com:47017'
    IP = IP_LAN if LAN else IP_PN   
    uri = 'mongodb://laocheng:laocheng@{0}'.format(IP)
    client = MongoClient(uri)
    return client

# 连接OCM1.5
def conn_ocm15(LAN=True):
    IP_LAN = '192.168.0.11:27017'
    IP_PN = 'www.parramountain.com:37017'
    IP = IP_LAN if LAN else IP_PN   
    uri = 'mongodb://liushizhan:liushizhan@{0}'.format(IP)
    client = MongoClient(uri)
    return client


def waimai_update(client, province='湖北省'):
    prov_mapp = yaml.load(open('E:\CloudStation\OCM\省份名单对应表.yaml', encoding='utf-8'))
    def sql_content(region, region_state):
        # 外卖月销量
        sql = [{'$match':{'lv3Name':region_state}},
               {'$group': {'_id' : region, '销量':{'$sum':'$monthSaleCount'}}}]
        return sql
    # 将省名转为英文名
    prov = prov_mapp[province]
    # 美团
    db_meituan = client15['WaimaiMeituan_201711']
    sql_region = sql_content('$lv3Name', {'$ne':None})
    cursor1 = db_meituan[prov].aggregate(sql_region)
    # 提取直辖县，比如神农架林区
    sql_city = sql_content('$lv2Name', None)
    cursor2 = db_meituan[prov].aggregate(sql_city)
    df = pd.DataFrame(list(cursor1))
    df_meituan = df.append(pd.DataFrame(list(cursor2))).set_index('_id')
    # 饿了么
    db_eleme = client15['Eleme_201711']
    sql_region = sql_content('$lv3Name', {'$ne':None})
    cursor1 = db_eleme[prov].aggregate(sql_region)
    sql_city = sql_content('$lv2Name', None)
    cursor2 = db_eleme[prov].aggregate(sql_city)
    df = pd.DataFrame(list(cursor1))
    df_eleme = df.append(pd.DataFrame(list(cursor2))).set_index('_id')
    # 百度外卖
    db_baidu = client15['WaimaiBaidu_201711']
    sql_region = sql_content('$lv3Name', {'$ne':None})
    cursor1 = db_baidu[prov].aggregate(sql_region)
    sql_city = sql_content('$lv2Name', None)
    cursor2 = db_baidu[prov].aggregate(sql_city)
    df = pd.DataFrame(list(cursor1))
    df_baidu = df.append(pd.DataFrame(list(cursor2))).set_index('_id')
    # 合并三大外卖
    df_merge = pd.concat([df_meituan, df_eleme, df_baidu], join='outer', axis=1)
    df_merge.columns = ['美团外卖', '饿了么外卖', '百度外卖']
    df_merge['外卖'] = df_merge.sum(axis=1)
    return df_merge

def factor_update(df_label, client, province='湖北省'):
    collection = client['Claudius']['TileData']
    group_dict = {'_id' : ''}
    for index in df_label.index:
        group_dict[index] = {'$sum' : '${0}.{1}'.format(df_label.loc[index,'一级标签'], df_label.loc[index,'sub'])}
    
    def sql_content(region, region_state):
        group_dict['_id'] = region
        sql = [
            {'$match':{'province':province, 'region':region_state, 'processTime':'201711', 'meters':1000}},
            {'$group' : group_dict},
            ]
        return sql

    # 提取区县通路点数,首先排除region为空的直辖县
    sql_region = sql_content('$region', {'$ne':None})
    cursor1 = collection.aggregate(sql_region)
    # 提取直辖县，比如神农架林区
    sql_city = sql_content('$city', None)
    cursor2 = collection.aggregate(sql_city)

    df = pd.DataFrame(list(cursor1))
    df = df.append(pd.DataFrame(list(cursor2)))
    return df.set_index('_id')

def factor_map_busi(df_factor, df_mapp):
    # 各区县通路点数乘以分配系数，解决一区多商圈的问题
    df_factor2 = df_factor.merge(df_mapp[['具体描述', '区县分配系数']], left_index=True,right_index=True,how='right')
    # 将控制填0
    df_factor2.fillna(0, inplace=True)
    df_factor2[df_factor.columns] = df_factor2[df_factor.columns].apply(lambda x: x * df_factor2['区县分配系数'])
    # 计算各商圈的因子点数
    df_factor3 = df_factor2.groupby('具体描述').sum().drop('区县分配系数', axis=1)
    
    return df_factor3

if __name__ == '__main__':
    path = '.'
    province='湖北省'
    filename = '%s各地区正负因子.xlsx'  % province

    client15 = conn_ocm15() #False)
    client16 = conn_ocm16() #False)
    xlsx = pd.ExcelFile(os.path.join(path, filename))
    # 读取地理因子信息标签，用于OCM取数
    df_label = pd.read_excel(xlsx, '地理因子信息', index_col='因子')
    isWaimai = False
    # 暂时去除外卖
    if '外卖' in df_label.index:
        isWaimai = True
        df_waimai = waimai_update(client15, province)
        df_label.drop(['外卖'], inplace=True)
    df_factor = factor_update(df_label, client16, province)
    if isWaimai:
        df_factor = pd.concat([df_factor, df_waimai['外卖']], axis=1)
    
    df_factor.to_excel('%s各区县地理因子数据.xlsx' % province)
    df_mapp = pd.read_excel('%sKSF各商圈区县分配系数.xlsx' % province, index_col='区县')
    df_factor2 = factor_map_busi(df_factor, df_mapp)
    # 将因子转化为整数
    df_factor2 = df_factor2.astype('int')
    df_factor2.to_excel('%s各商圈地理因子数据.xlsx' % province)
    
    
    
    


# -*- coding: utf-8 -*-
"""
Created on Sun Apr 15 22:01:21 2018

@author: jasonai

以LinxData数据为例，解释定位高德坐标的关键点：
地址匹配使用高德匹配和百度匹配两种匹配方式，高德返回的经纬度值转换为百度标准的经纬度
高德使用三种匹配方式，三种方式都会返回定位点的地址信息、经纬度、定位的地址类型，
地址类型按精确度依次为兴趣点、道路交叉路口、道路、区县
1、省市区+附近标志+门店名
优点：精确定位。 缺点：准确度不高，容易定位到其他地址的同名门店
2、省市区+附近标志
优点：附近标志多为道路交叉口描述，高德容易模糊匹配出交叉口，有时也会精确定位到门店
缺点：大部分情况下只能定位到交叉口，经纬度有偏差
3、省市区+道路（使用正则从附近标志词条中拆分出两条道路名，未能拆分的用原始数据的路名列值替代）
返回的经纬度用来检验方式2的返回值
百度匹配与高德第三种方式相同，因为百度模糊匹配准确度不高，所以只采用“省市区+道路”的方式，
返回的经纬度与高德方式3的返回值对比

返回经纬度距离对比，返回两点之间的距离
OCM-高德1、OCM-高德2、高德1-高德2、高德2-高德3、高德3-百度
OCM-高德1、OCM-高德2分别用来验证高德1、高德2的准确度
高德1-高德2的值小于1000米，则以高德1为准，若高德1-高德2的值大于1000米，则用高德2-高德3、
高德3-百度并结合地址类型判断是否选择高德2
"""
import os
import sys
import json
import urllib
import math
import requests
import numpy as np
import pandas as pd
from pmipy import pmi
from tqdm import tqdm
from random import choice
from threading import Thread


gd_url = 'http://restapi.amap.com/v3/geocode/geo?' #高德地理编码api服务地址
keys_list = ['f2028746f4fe36d6b665d626b9481e73', 'd915aa691a88cdb29fe18f120feea269', 
'9ee6aa4c8210ff9d8134a16d5d216053','1a7169154454db57564bd1639d849dcb',
'7a6c90d70d50f22d942928d1ffd25d8a','b77b9f05cdb10aac5b5061e31cd24254',
'8b55981ffe68fed09b3299118955b317','2c603a0aa3f9ac69afe468cd8cebfda5',]

x_pi = 3.14159265358979324 * 3000.0 / 180.0
pi = 3.1415926535897932384626  # π
a = 6378245.0  # 长半轴
ee = 0.00669342162296594323  # 偏心率平方


class Geocoding:
    def __init__(self, api_key):
        self.api_key = api_key

    def geocode(self, address):
        """
        利用高德geocoding服务解析地址获取位置坐标
        :param address:需要解析的地址
        :return:
        """
        geocoding = {'s': 'rsv3',
                     'key': self.api_key,
                     'city': '全国',
                     'address': address}
        geocoding = urllib.urlencode(geocoding)
        ret = urllib.urlopen("%s?%s" % ("http://restapi.amap.com/v3/geocode/geo", geocoding))

        if ret.getcode() == 200:
            res = ret.read()
            json_obj = json.loads(res)
            if json_obj['status'] == '1' and int(json_obj['count']) >= 1:
                geocodes = json_obj['geocodes'][0]
                lng = float(geocodes.get('location').split(',')[0])
                lat = float(geocodes.get('location').split(',')[1])
                return [lng, lat]
            else:
                return None
        else:
            return None


def gcj02_to_bd09(lng, lat):
    """
    火星坐标系(GCJ-02)转百度坐标系(BD-09)
    谷歌、高德——>百度
    :param lng:火星坐标经度
    :param lat:火星坐标纬度
    :return:
    """
    z = math.sqrt(lng * lng + lat * lat) + 0.00002 * math.sin(lat * x_pi)
    theta = math.atan2(lat, lng) + 0.000003 * math.cos(lng * x_pi)
    bd_lng = z * math.cos(theta) + 0.0065
    bd_lat = z * math.sin(theta) + 0.006
    return [bd_lng, bd_lat]


def bd09_to_gcj02(bd_lon, bd_lat):
    """
    百度坐标系(BD-09)转火星坐标系(GCJ-02)
    百度——>谷歌、高德
    :param bd_lat:百度坐标纬度
    :param bd_lon:百度坐标经度
    :return:转换后的坐标列表形式
    """
    x = bd_lon - 0.0065
    y = bd_lat - 0.006
    z = math.sqrt(x * x + y * y) - 0.00002 * math.sin(y * x_pi)
    theta = math.atan2(y, x) - 0.000003 * math.cos(x * x_pi)
    gg_lng = z * math.cos(theta)
    gg_lat = z * math.sin(theta)
    return [gg_lng, gg_lat]


def wgs84_to_gcj02(lng, lat):
    """
    WGS84转GCJ02(火星坐标系)
    :param lng:WGS84坐标系的经度
    :param lat:WGS84坐标系的纬度
    :return:
    """
    if out_of_china(lng, lat):  # 判断是否在国内
        return lng, lat
    dlat = _transformlat(lng - 105.0, lat - 35.0)
    dlng = _transformlng(lng - 105.0, lat - 35.0)
    radlat = lat / 180.0 * pi
    magic = math.sin(radlat)
    magic = 1 - ee * magic * magic
    sqrtmagic = math.sqrt(magic)
    dlat = (dlat * 180.0) / ((a * (1 - ee)) / (magic * sqrtmagic) * pi)
    dlng = (dlng * 180.0) / (a / sqrtmagic * math.cos(radlat) * pi)
    mglat = lat + dlat
    mglng = lng + dlng
    return [mglng, mglat]


def gcj02_to_wgs84(lng, lat):
    """
    GCJ02(火星坐标系)转GPS84
    :param lng:火星坐标系的经度
    :param lat:火星坐标系纬度
    :return:
    """
    if out_of_china(lng, lat):
        return lng, lat
    dlat = _transformlat(lng - 105.0, lat - 35.0)
    dlng = _transformlng(lng - 105.0, lat - 35.0)
    radlat = lat / 180.0 * pi
    magic = math.sin(radlat)
    magic = 1 - ee * magic * magic
    sqrtmagic = math.sqrt(magic)
    dlat = (dlat * 180.0) / ((a * (1 - ee)) / (magic * sqrtmagic) * pi)
    dlng = (dlng * 180.0) / (a / sqrtmagic * math.cos(radlat) * pi)
    mglat = lat + dlat
    mglng = lng + dlng
    return [lng * 2 - mglng, lat * 2 - mglat]


def bd09_to_wgs84(bd_lon, bd_lat):
    lon, lat = bd09_to_gcj02(bd_lon, bd_lat)
    return gcj02_to_wgs84(lon, lat)


def wgs84_to_bd09(lon, lat):
    lon, lat = wgs84_to_gcj02(lon, lat)
    return gcj02_to_bd09(lon, lat)


def _transformlat(lng, lat):
    ret = -100.0 + 2.0 * lng + 3.0 * lat + 0.2 * lat * lat + \
          0.1 * lng * lat + 0.2 * math.sqrt(math.fabs(lng))
    ret += (20.0 * math.sin(6.0 * lng * pi) + 20.0 *
            math.sin(2.0 * lng * pi)) * 2.0 / 3.0
    ret += (20.0 * math.sin(lat * pi) + 40.0 *
            math.sin(lat / 3.0 * pi)) * 2.0 / 3.0
    ret += (160.0 * math.sin(lat / 12.0 * pi) + 320 *
            math.sin(lat * pi / 30.0)) * 2.0 / 3.0
    return ret


def _transformlng(lng, lat):
    ret = 300.0 + lng + 2.0 * lat + 0.1 * lng * lng + \
          0.1 * lng * lat + 0.1 * math.sqrt(math.fabs(lng))
    ret += (20.0 * math.sin(6.0 * lng * pi) + 20.0 *
            math.sin(2.0 * lng * pi)) * 2.0 / 3.0
    ret += (20.0 * math.sin(lng * pi) + 40.0 *
            math.sin(lng / 3.0 * pi)) * 2.0 / 3.0
    ret += (150.0 * math.sin(lng / 12.0 * pi) + 300.0 *
            math.sin(lng / 30.0 * pi)) * 2.0 / 3.0
    return ret


def out_of_china(lng, lat):
    """
    判断是否在国内，不在国内不做偏移
    :param lng:
    :param lat:
    :return:
    """
    return not (lng > 73.66 and lng < 135.05 and lat > 3.86 and lat < 53.55)


#======== 以下进行高德地址编码====================================
# 第一次返回结果，很有可能包含网络无响应的数据;checkResult装饰器可判断网络相应情况，如无响应则继续循环执行gd_geocode函数
@pmi.checkResult(keyword=2.0, loopTime=50) 
@pmi.checkResult(keyword=1.0, loopTime=100) 
def gd_geocode(address): #根据地址获取高德经纬度后转化为百度经纬度\
    # 解决高德API日访问次数限制问题
    key = choice(keys_list)
    gd_parm1 = {'key': key, 'address': address, 'city': '',} #高德key
    try:
        response = requests.get(url=gd_url, params=gd_parm1, timeout=10).json()
        if response['status'] == '1':
            # 获取第一个返回地址信息
            geocode = response['geocodes'][0]
            lng = float(geocode['location'].split(',')[0])
            lat = float(geocode['location'].split(',')[1])
            f_address = geocode['formatted_address']
            province = geocode['province']
            city = geocode['city']
            district = geocode['district']
            level = geocode['level']
            result = [lng, lat, f_address, province, city, district, level]
        else:
            result = [2.0,2.0,'','','','',''] # 请求失败使用2.0表示
    except:
        result = [1.0,1.0,'','','','',''] # 网络无响应使用1.0表示
    print('===>',result)
    return result

def run_thread(df, adr_search):
    res_list = []
    for adr in df[list(adr_search)].sum(axis=1):
        res_list.append(gd_geocode(adr))
    df2 = pd.DataFrame(res_list,columns=['经度','纬度','返回地址','省','市','区','地址类型'],index=df.index)
    # df[['new1', 'new2', 'new3', 'new4','new5', 'new6','new7']] = df.apply(gd_geocode)
    return pd.concat([df,df2],axis=1)


# 多线程
class MyThread(Thread):
    def __init__(self, df, adr_search):
        Thread.__init__(self)
        self.df = df
        self.adr_search = adr_search

    def run(self):
        self.result = run_thread(self.df, self.adr_search)

    def get_result(self):
        return self.result


def transform(df, lnglat, initial_coordinate, target_coordinate):
    res_list =[]
    for lng, lat in tqdm(df[list(lnglat)].values):
        if initial_coordinate == 'gaode':
            if target_coordinate == 'baidu': # 高德转百度
                res_list.append(gcj02_to_bd09(lng, lat))
            elif target_coordinate == 'global': # 高德转国际
                res_list.append(gcj02_to_wgs84(lng, lat))
            else:
                sys.exit('{0}转{1}尚未开发，如有需求，请邮件jasonai！'.format(initial_coordinate, target_coordinate))
        elif initial_coordinate == 'baidu':
            if target_coordinate == 'gaode': # 百度转高德
                res_list.append(bd09_to_gcj02(lng, lat))
            elif target_coordinate == 'global': # 百度转国际
                res_list.append(bd09_to_wgs84(lng, lat))
            else:
                sys.exit('{0}转{1}尚未开发，如有需求，请邮件jasonai！'.format(initial_coordinate, target_coordinate))
        elif initial_coordinate == 'global':
            if target_coordinate == 'gaode': # 国际转高德
                res_list.append(wgs84_to_gcj02(lng, lat))
            elif target_coordinate == 'baidu': # 国际转百度
                res_list.append(wgs84_to_bd09(lng, lat))
            else:
                sys.exit('{0}转{1}尚未开发，如有需求，请邮件jasonai！'.format(initial_coordinate, target_coordinate))
        else:
            sys.exit('{0}转{1}尚未开发，如有需求，请邮件jasonai！'.format(initial_coordinate, target_coordinate))
    res_df = pd.DataFrame(res_list, columns=[target_coordinate+'经度',target_coordinate+'纬度'],index=df.index)
    
    return pd.concat([df, res_df], axis=1)
   
    
@pmi.execInfo()
def coordinateTransform(filepath, file_ouput, lnglat, initial_coordinate, target_coordinate):
    path = os.path.split(filepath)[0]
    filename = os.path.splitext(os.path.split(filepath)[1])[0]
    filesuffix = os.path.splitext(os.path.split(filepath)[1])[1]
    # 读取文件
    if filesuffix == '.csv':
        df = pd.read_csv(filepath, engine='python')
    elif filesuffix == '.xlsx':
        df = pd.read_excel(filepath)
    else:
        sys.exit('Please input csv or xlsx file!')
    
    df_res = transform(df, lnglat, initial_coordinate, target_coordinate)
    if file_ouput:
        filename = file_ouput
    outfile = filename + '_转' + target_coordinate + '坐标.xlsx'
    df_res.to_excel(os.path.join(path, outfile), index=False)
    

@pmi.execInfo()
def getTargetCoordinate(filepath, adr_search, thread=20, target_coordinate='gaode',file_ouput=''):
    path = os.path.split(filepath)[0]
    filename = os.path.splitext(os.path.split(filepath)[1])[0]
    filesuffix = os.path.splitext(os.path.split(filepath)[1])[1]
    # 读取文件
    if filesuffix == '.csv':
        df = pd.read_csv(filepath, engine='python')
    elif filesuffix == '.xlsx':
        df = pd.read_excel(filepath)
    else:
        sys.exit('Please input csv or xlsx file!')
    
    try:
        thread = int(thread)
    except ValueError:
        sys.exit('请输入正确的线程数(整数值)！')
    # df = df.head(50)
    # 如果线程数小于数据框长度的1/2
    if thread < len(df) / 2:
        df_split = np.array_split(df, thread)
    else:
        df_split = np.array_split(df, 2)

    # 构建多线程
    thd_list = []
    for df in df_split:
        thd = MyThread(df, adr_search)
        thd.start()
        thd_list.append(thd)
    
    for t in thd_list: # 等待所有线程执行完毕
        t.join()
    
    res_list = []
    for t in thd_list:
        res = t.get_result()
        # print(t.get_result())
        res_list.append(res)
    
    df_res = pd.concat(res_list) 
    
    if target_coordinate != 'gaode':
        df_res = transform(df_res, ['经度','纬度'], 'gaode', target_coordinate)
    if file_ouput:
        filename = file_ouput
    outfile = filename + '_' + target_coordinate + '坐标.xlsx'
    df_res.to_excel(os.path.join(path, outfile), index=False)


if __name__ == '__main__':
    filepath = r'E:\CloudStation\康饮商圈分析\零售饮料\江苏省linxdata_经纬度匹配最终版V1.xlsx'
    adr_search = ['省份','城市名称','附近标志']
    getTargetCoordinate(filepath, adr_search)
    


# -*- coding: utf-8 -*-
"""
Created on Mon Apr  9 22:04:45 2018

@author: jasonai
"""

import sys
import math
from pmipy import pmi
import numpy as np
import pandas as pd
from tqdm import tqdm
import multiprocessing as mp

def haversine(lng1, lat1, lng2, lat2): # 根据经纬度计算两点距离  
    lng1, lat1, lng2, lat2 = map(math.radians, [lng1, lat1, lng2, lat2])  
    dlng = lng2 - lng1   
    dlat = lat2 - lat1   
    a = math.sin(dlat/2)**2 + math.cos(lat1) * math.cos(lat2) * math.sin(dlng/2)**2 
    c = 2 * math.asin(math.sqrt(a))   
    r = 6371 # 地球平均半径，单位为公里  
    d = c * r * 1000
    return d

# 此函数速度太慢，放弃使用
# 需输入点位经纬度数据，宜出行数据，其中宜出行的数据表头格式是标准格式
# df_ycx['distance'] = df_ycx.apply(lambda row: haversine(lng1, lat1,row['lng'], row['lat']), axis=1)此代码更耗时
def yichuxing_backup1(df_point, df_ycx, lnglat, meter=500):
    PF = []
    for lng1, lat1 in tqdm(df_point.loc[:, [lnglat]].values):
        peo_flow = 0 # 人流指数
        for lng2, lat2, count in df_ycx.loc[:,['lng', 'lat', 'count']].values:
            if haversine(lng1, lat1, lng2, lat2) <= meter:
                peo_flow += count
        PF.append(peo_flow)
        
    df_point['人流指数'] = PF
    return df_point

def run_task(df,df_ycx, lnglat, meter):
    PF = []
    for lng1, lat1 in tqdm(df.loc[:, lnglat].values):
        peo_flow = 0 # 人流指数
        for lng2, lat2, count in df_ycx.loc[:,['lng', 'lat', 'count']].values:
            #print(peo_flow)

            if haversine(lng1, lat1, lng2, lat2) <= meter:
                peo_flow += count
        PF.append(peo_flow)
    df['人流指数'] = PF
    return df
    #print('Task {0} end.'.format(name))

def yichuxing_backup2(df_point, df_ycx, lng, lat, meter, ncore=6):
    def tmp_func(df):
        PF = []
        for lng1, lat1 in tqdm(df.loc[:, [lng, lat]].values):
            peo_flow = 0 # 人流指数
            print(1)
            for lng2, lat2, count in df_ycx.loc[:,['lng', 'lat', 'count']].values:
                if haversine(lng1, lat1, lng2, lat2) <= meter:
                    peo_flow += count
            PF.append(peo_flow)
        df['人流指数'] = PF
        return df

    def apply_parallel(df_grouped, func):
        """利用 Parallel 和 delayed 函数实现并行运算"""
        results = Parallel(n_jobs=-1)(delayed(func)(group) for name, group in df_grouped)
        return pd.concat(results)
    nrow = len(df_point)
    df_point['grouping'] = np.arange(nrow) // (nrow / ncore)
    df_grouped = df_point.groupby('grouping')
    df_point =apply_parallel(df_grouped, tmp_func)
    return df_point

def yichuxing_backup3(df_point, df_ycx, lng, lat, meter, ncore=2):
    def proc(df):
        PF = []
        for lng1, lat1 in tqdm(df.loc[:, [lng, lat]].values):
            peo_flow = 0 # 人流指数
            # print(1)
            for lng2, lat2, count in df_ycx.loc[:,['lng', 'lat', 'count']].values:
                if haversine(lng1, lat1, lng2, lat2) <= meter:
                    peo_flow += count
            PF.append(peo_flow)
        df['人流指数'] = PF
        return df
    
    p = mp.Pool(ncore)
    split_dfs = np.array_split(df_point, ncore)
    pool_results = p.map(proc, split_dfs)
    p.close()
    p.join()
    df_all = pd.concat(pool_results)
    # pdt.assert_series_equal(parts['id'], big_df['id'])
    return df_all

@pmi.execInfo()
def getPeopleFlow(pointFile, ycxFile, lnglat=('经度','纬度'), meter='500', core_num='max', outputFile=''):
    # 读取店铺点数文件
    if pointFile[-4:] == '.csv':
        df_point = pd.read_csv(pointFile, engine='python')
    elif pointFile[-5:] == '.xlsx':
        df_point = pd.read_excel(pointFile)
    else:
        sys.exit('Please input csv or xlsx file!')
    # 读取宜出行数据文件，获取人流
    if ycxFile[-4:] == '.csv':
        df_ycx = pd.read_csv(ycxFile, engine='python')
    elif ycxFile[-5:] == '.xlsx':
        df_ycx = pd.read_excel(ycxFile)
    else:
        sys.exit('Please input csv or xlsx file!')
    
    # 输出文件名
    if not outputFile:
        outFile = pointFile + '_' + ycxFile + '_{}'.format(meter)
        outFile = outFile.replace('.csv', '')
        outFile = outFile.replace('.xlsx', '')
    # 获取经纬度表头列表
    lnglat = list(lnglat)
    meter = int(meter)
    # 确定并发使用的CPU线程数
    core_max = mp.cpu_count()
    if core_num != 'max':
        try:
            ncore = int(core_num)
            core_num = ncore if ncore <= core_max else core_max
        except ValueError:
            sys.exit('请输入正确的线程数(整数值)！')
    else:
        core_num = core_max
    
    df_split = np.array_split(df_point, core_num)
    p = mp.Pool(processes=core_num)
    funcList = []
    for df in df_split:
        f = p.apply_async(run_task, args=(df, df_ycx, lnglat, meter,))
        funcList.append(f)
    print('Waiting for all subprocesses done...')
    p.close()
    p.join()
    resList = []
    for f in funcList:
        resList.append(f.get(timeout=0.5))
    df_all = pd.concat(resList)
    # 添加时间信息
    df_all['爬取时间'] = ycxFile.split('.')[0][-19:-3] # '南京2018-04-05-11-16-34.csv'格式
    df_all.to_excel(outFile+'.xlsx', index=False)
    print('All processes done!')


if __name__ == '__main__':
    pointFile = '江苏省linxdata_经纬度匹配最终版V1.xlsx'
    ycxFile = '南京2018-04-01-17-40-57.csv'
    getPeopleFlow(pointFile, ycxFile, lnglat, 500, 12, 'ts')
    
    
    


"""
Created on Sat Apr 21 07:29:46 2018

@author: jasonai
"""

import logging


def createLogger(name):
    # 获取logger实例，如果参数为空则返回root logger
    logger = logging.getLogger(name)
    
    # 指定logger输出格式
    formatter = logging.Formatter('[%(asctime)s][%(module)s][line:%(lineno)d][%(levelname)s]: %(message)s')
    
    # 文件日志
    file_handler = logging.FileHandler(name + ".log")
    file_handler.setFormatter(formatter)  # 可以通过setFormatter指定输出格式
    
    # 控制台日志
    console_handler = logging.StreamHandler() #sys.stdout)
    console_handler.formatter = formatter  # 也可以直接给formatter赋值
     
    # 为logger添加的日志处理器
    logger.addHandler(file_handler)
    logger.addHandler(console_handler)
     
    # 指定日志的最低输出级别，默认为WARN级别
    logger.setLevel(logging.INFO)
    return logger


def createCustomLogger(name):
    formatter = logging.Formatter(fmt='[%(asctime)s][%(levelname)s][%(module)s] %(message)s', 
                                        datefmt='%m/%d %I:%M:%S%p')

    handler = logging.StreamHandler()
    handler.setFormatter(formatter)

    logger = logging.getLogger(name)
    logger.setLevel(logging.DEBUG)
    logger.addHandler(handler)
    return logger





# -*- coding: utf-8 -*-
"""
Created on Fri Apr  6 09:59:36 2018

@author: jasonai
"""

import numpy  as np
import pandas as pd
from sklearn.decomposition import PCA
from sklearn.preprocessing import StandardScaler
from sklearn.linear_model import LinearRegression
# from sklearn.model_selection import train_test_split
from sklearn.metrics import r2_score #, mean_squared_error


# PCA降维
def proc_pca(df):
    # 数据标准化
    # df = df_merge
    df.fillna(0, inplace=True)
    scaler = StandardScaler().fit(df.values)  # sklearn的标准化类

    """
    暂时不做数据拆分
    # random_state：可以为整数、RandomState实例或None，默认为None
    # 若为None时，每次生成的数据都是随机，可能不一样,若为整数时，每次生成的数据都相同
    train_data, test_data = train_test_split(df.values, test_size=0.3)
    X_train, y_train = scaler.transform(train_data)[:, :-1], scaler.transform(train_data)[:, -1]
    X_test, y_test = scaler.transform(test_data)[:, :-1], scaler.transform(test_data)[:, -1]
    """

    X_train, y_train = scaler.transform(df.values)[:, :-1], scaler.transform(df.values)[:, -1]
    # 主成分分析
    pca = PCA(n_components=2)
    X_train_pca = pca.fit_transform(X_train)
    # 线性回归
    linreg = LinearRegression()
    linreg.fit(X_train_pca, y_train)
    y_pred = linreg.predict(X_train_pca)
    R2 = r2_score(y_train, y_pred)
    # 预估与实际的比值
    mean = scaler.mean_  # 平均值，原真实数据用
    std = np.sqrt(scaler.var_)  # 标准差，还原真实数据用
    y_pred_restore = y_pred * std[-1] + mean[-1]
    predvsReal = y_pred_restore / df.values[:, -1]

    return predvsReal, R2

def proc_onlyLnr(df):
    # 数据标准化
    df.fillna(0, inplace=True)
    scaler = StandardScaler().fit(df.values)  # sklearn的标准化类
    
    X_train, y_train = scaler.transform(df.values)[:, :-1], scaler.transform(df.values)[:, -1]

    # 线性回归
    linreg = LinearRegression()
    linreg.fit(X_train, y_train)
    y_pred = linreg.predict(X_train)
    R2 = r2_score(y_train, y_pred); print(R2)
    # 预估与实际的比值
    mean = scaler.mean_  # 平均值，原真实数据用
    std = np.sqrt(scaler.var_)  # 标准差，还原真实数据用
    y_pred_restore = y_pred * std[-1] + mean[-1]
    predvsReal = y_pred_restore / df.values[:, -1]
    return predvsReal, R2

def main(province='湖北省', period=3):
    df_factor = pd.read_excel('%s各商圈地理因子数据.xlsx' % province, index_col=0)
    df_sale = pd.read_excel('%sKSF各商圈TT&MT出货.xlsx' % province, index_col=0)
    df_factor['学校'] = df_factor[['小学', '中学', '高等院校']].sum(axis=1)
    category = [i+'.{}'.format(period-1) for i in ['容器面', '高价袋','中价面','平低价','干脆面']]
    df_sale = df_sale[category]
    df_sale['整体.{}'.format(period-1)] = df_sale.sum(axis=1)
    df_merge = pd.concat([df_factor, df_sale['整体.{}'.format(period-1)]], axis=1)
    """
    # 只考虑8个筛选过的正负因子，最终R方为0.86,小于全部因子的0.91，补位使用
    # 获取相应省份的正负因子用
    df_mapt = pd.read_excel('%s各地区正负因子.xlsx' % province, sheetname='省份-地区对应表', index_col=0)
    df_pninfo = pd.read_excel('%s各地区正负因子.xlsx' % province, sheetname='各地区正负因子系数', index_col=0)
    pn_list = list(df_pninfo.loc[df_mapt.loc[province]]['整体'])
    df_factor2 = df_factor[pn_list]
    df_merge2 = pd.concat([df_factor2, df_sale['整体.{}'.format(period-1)]], axis=1)
    """
    predvsReal, R2 = proc_pca(df_merge)

    


# -*- coding: utf-8 -*-
"""
Created on Wed Mar 28 20:40:39 2018

@author: jasonai
"""

import os
import re
import collections
import pandas as pd
from pymongo import MongoClient

# 连接OCM1.6
def conn_ocm16(LAN=True):
    IP_LAN = '192.168.0.21:27017'
    IP_PN = 'www.parramountain.com:47017'
    IP = IP_LAN if LAN else IP_PN   
    uri = 'mongodb://laocheng:laocheng@{0}'.format(IP)
    client = MongoClient(uri)
    db = client['Claudius']
    collection = db['TileData']
    return collection

# 连接OCM1.5
def conn_ocm15(LAN=True):
    IP_LAN = '192.168.0.11:27017'
    IP_PN = 'www.parramountain.com:37017'
    IP = IP_LAN if LAN else IP_PN   
    uri = 'mongodb://liushizhan:liushizhan@{0}'.format(IP)
    client = MongoClient(uri)
    db = client['Claudius3']
    collection = db['TileData']
    return collection



def busi_mapping(province = '湖北省'):
    # 获取指定省份下的商圈名单
    try:
        busiTitle = '具体描述'
        df1 = pd.read_excel('%sKSF各商圈TT&MT出货.xlsx' % province, usecols=[busiTitle])
    except ValueError:
        busiTitle = '商圈'
        df1 = pd.read_excel('%sKSF各商圈TT&MT出货.xlsx' % province, usecols=[busiTitle])
    
    # 指定省份各区县地理因子
    df2 = pd.read_excel('%s各区县特通点数及人口数据.xlsx' % province, index_col='区县')
    
    # 存放OCM区县完整名单
    county_full = collections.Counter(df2.index)
    # 存放OCM区县去尾（市区县）名单
    list_cs = [i[:-1] for i in df2.index]
    county_simple = collections.Counter(list_cs)
    # 存放商圈信息的字典
    busi_dict = {}
    for busi in df1.values[:, 0]:
        district_list = re.split(r'[、，,()（）]', busi)
        # 商圈包含的区县信息
        temp_list = []
        info_list = []
        for index, district in enumerate(district_list):
            # 排除包含'市辖'的字段或空字段
            if ('市辖' not in district) and (district != ''):
                # mapping商圈中提取得到的区县
                if county_full[district] == 1:
                    temp_list.append(district)
                    info_list.append('匹配')
                # 判断是否为县改区或市
                elif county_simple[district[:-1]] == 1:
                    # 获取区县字段所在位置
                    county_index = list_cs.index(district[:-1])
                    temp_list.append(df2.index[county_index])
                    info_list.append('可能级别变动')
                elif (county_full[district]>1) or (county_simple[district]>1) :
                    info_list.append('区县名重复')
                else:
                    info_list.append('匹配失败')
        if ('区县名重复' in info_list) or ('匹配失败' in info_list):
            info2 = '失败'
        else:
            info2 = '成功'
        busi_dict[busi] = [temp_list, info_list, info2]
    df3 = pd.DataFrame(busi_dict).T
    df3.columns = ['匹配区县', '信息描述', '匹配状态']
    # df3.to_excel('%s商圈与OCM区县匹配结果.xlsx' % province, index_label=busiTitle)、
    return df3


df busi_mapping()









import os
import sys
import re
from setuptools import setup

# long_description: Take from README file
with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as f:
    readme = f.read()

# Version Number
version = '0.12.23'

def check_dependencies():
    requirements = [line.strip() for line in open('requirements.txt')]

    # Just make sure dependencies exist, I haven't rigorously
    # tested what the minimal versions that will work are
    # (help on that would be awesome)
    try:
        import pandas
    except ImportError:
        requirements.append('pandas')

    return requirements

# Dependencies
"""
if sys.platform.startswith('win'):
    requirements = [line.strip() for line in open('requirements.txt')]
else:
    raise OSError("目前只支持windows系统.")
"""
# This shouldn't be necessary anymore as we dropped official support for < 2.7 and < 3.3
if (sys.version_info[0] == 2 and sys.version_info[:2] < (2, 7)) or (sys.version_info[0] == 3 and sys.version_info[:2] < (3, 2)):
    requirements = requirements + []

requirements = check_dependencies()

setup(
    name='pmipy',
    version=version,
    license='MIT',
    author='biojim',
    author_email='biojxz@163.com',
    description='A tool for my team.',
    long_description=readme,
    packages=['pmipy'],
    package_data={'pmipy': ['tests/*.png', 'tests/*.py', 'Rscript/*.r', 'DemandPredict/*.py', 'BITauto/*.py', 'xlwings.applescript'],},
    keywords=["I don't want to tell you!"],
    install_requires=requirements,
    entry_points={'console_scripts': ['pmipy=pmipy.command_line:main'],},
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: MacOS',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Topic :: Office/Business :: Financial :: Spreadsheet',
        'License :: OSI Approved :: BSD License'],
    platforms=['Windows', "Mac OS-X"],
)



# OCMVisualization

import os


def run_Rscript(data_path, tag, path):
    if not os.path.exists(tag):
        os.mkdir(tag)
    print(os.getcwd())
    Rscript_path = os.path.join(path, 'Rscript', 'OCM_feature.r');print(Rscript_path)
    # 运行R程序，给文件路径加上双引号，解决文件夹名含空格的问题
    os.system("""Rscript "%s" %s %s""" % (Rscript_path, data_path, tag))




import numpy as np
import pandas as pd
import os, sys
import seaborn as sns
import matplotlib as mpl
import matplotlib.pyplot as plt
from pyplotz.pyplotz import PyplotZ
from sklearn.preprocessing import MinMaxScaler


def interpreter(time_tag, sale_tag, sales_file_proc, province):
    zhfont1 = mpl.font_manager.FontProperties(fname='C:\Windows\Fonts\simkai.ttf')
    df = pd.read_excel(time_tag + "/%s_median.xlsx" % time_tag, index_col=0)
    # 去除训练次数，投票等信息
    df = df.loc[:, [sale_tag, '预估销量', '预估与实际比值']]
    try:
        df_grading = pd.read_excel(sales_file_proc, usecols=[0, 1], index_col=0)
    except Exception as err:
        print(err)  # 输出报错的原因
    df = df.merge(df_grading, left_index=True, right_index=True, how="left")  # 添加商圈分级信息，可能会产生缺失值

    """
    业绩结合分级的调整方案：
    # 计算每个商圈等级的中位数销量，使用中位数来判断商圈是否需要微调
    df_t1 = df[df['客户形态对应的商圈等级'].notnull()].loc[:, ['客户形态对应的商圈等级', sale_tag]]  # 去掉缺失等级信息的商圈
    df_grading_sale = df_t1.groupby('客户形态对应的商圈等级').median()
    grading_sale = df_grading_sale.to_dict()
    # 添加微调信息
    # df.columns: ['整体平均千箱', '预估销量', '预估与实际比值', '客户形态对应的商圈等级']
    suggest_list = []
    df.fillna('未知分级', inplace=True)
    for row in df.values:
        if row[3] != '未知分级':  # 客户形态对应的商圈等级非空缺时
            if row[2] >= 1.5:  # 先筛选出预估与实际比值大于等于1.5的商圈
                if row[3] == '1级':
                    suggest = '建议维持'
                else:
                    grading = str(int(row[3][0]) - 1) + row[3][1]
                    gs = grading_sale[sale_tag][grading]
                    suggest = '建议上调' if row[1] >= gs else '建议维持'
            elif row[2] <= 0.75:  # 筛选下调的第一步
                if row[3] == '6级':
                    suggest = '建议维持'
                else:
                    grading = str(int(row[3][0]) + 1) + row[3][1]
                    gs = grading_sale[sale_tag][grading]
                    gs2 = gs * 1.2  # 湖北省由于下调的商圈太少，我们将中位值提高20%
                    suggest = '建议下调' if row[1] <= gs2 else '建议维持'
            else:
                suggest = '建议维持'
        else:
            suggest = '信息缺失'
        suggest_list.append(suggest)
        """
    df['业绩表现'] = np.where(df['预估与实际比值'] > 1.5, '业绩还有上升空间', '业绩良好')
    # 为客户准备excel表
    df.to_excel(time_tag + "/%s_综合.xlsx" % time_tag)
    # 对df[sale_tag]列进行归一化而不是标准化
    df['sales_avg_norm'] = MinMaxScaler().fit_transform(df.loc[:, [sale_tag, '预估与实际比值']])[:, 0]
    df_maintain = df[(df['预估与实际比值'] > 0.9) & (df['预估与实际比值'] < 1.1)]
    df_up = df[df['业绩表现'] == '业绩还有上升空间']
    # df_down = df[df['业绩表现'] == '建议下调']
    df_factor = pd.read_excel(time_tag + '/%s_正负因子.xlsx' % time_tag)
    df_ocm = pd.read_excel('%s商圈OCM_%s.xlsx' % (province, sale_tag), index_col=0)
    df_ocm.fillna(0, inplace=True)
    param = list(df_factor["参数"].values)
    df_ocm_norm = MinMaxScaler().fit_transform(df_ocm.loc[:, param])
    df_ocm_norm = pd.DataFrame(df_ocm_norm, index=df_ocm.index, columns=param)

    def data_process(df_ocm_norm, df):
        df_concat = pd.concat([df_ocm_norm, df['sales_avg_norm']], axis=1, join='inner')
        df_proc = df_concat.apply(lambda x: x / df_concat['sales_avg_norm'])  # 标准化后，将各因子除以标准化的平均销量
        df_proc.drop('sales_avg_norm', axis=1, inplace=True)
        return df_proc

    df_maintain_proc = data_process(df_ocm_norm, df_maintain)
    df_maintain_proc.reset_index(inplace=True)
    for index, i in enumerate(param):
        df_maintain_proc['参数'] = i
        df_maintain_proc2 = df_maintain_proc.loc[:, ["商圈", '参数', i]]
        df_maintain_proc2.columns = ['商圈', '参数', 'value']
        # 去除极端异常值
        df_maintain_proc2 = df_maintain_proc2[(df_maintain_proc2['value'] < 8) & (df_maintain_proc2['value'] > -8)]

        df_maintain_proc3 = df_maintain_proc2 if not index \
            else df_maintain_proc3.append(df_maintain_proc2, ignore_index=True)

    df_up_proc = data_process(df_ocm_norm, df_up)
    # df_down_proc = data_process(df_ocm_norm, df_down)

    def data_visual(df, df_maintain, param, tag):
        sns.set_style("white")
        for i in df.index:
            pltz = PyplotZ()
            pltz.enable_chinese()
            plt.xticks(rotation='30', fontproperties=zhfont1)
            plt.plot((np.ones(10)) * 4.5, np.arange(-4, 6), linestyle="--")  # 绘制分割正负因子的虚线
            plt.scatter(np.arange(len(param)), df.loc[i].values, c='#CD2626', marker='*')  # 横坐标用len(param)代替param
            df_maintain2 = df_maintain
            sns.boxplot(x='参数', y='value', data=df_maintain2, fliersize=1.3, color="#48D1CC")
            pltz.xlabel('地理环境因子')
            pltz.title('%s_%s' % (i, tag))
            plt.savefig(os.path.join(time_tag, '%s_%s.png' % (i, tag)), bbox_inches='tight')
            plt.close()

    data_visual(df_up_proc, df_maintain_proc3, param, '业绩还有上升空间')
    # data_visual(df_down_proc, df_maintain_proc3, param, '建议下调')
    df_maintain_proc.drop(columns=['参数'], inplace=True)
    df_maintain_proc.set_index("商圈", inplace=True)
    df_final = pd.concat([df_maintain_proc, df_up_proc])  # df_down_proc,
    df_final = pd.concat([df_final, df.loc[:, ['sales_avg_norm', '业绩表现']]], axis=1, join='inner')
    df_final.to_excel(time_tag + "/%s_参数销量比.xlsx" % time_tag)


if __name__ == "__main__":
    time_tag = '整体平均千箱_2018-01-05-16h03m'
    sale_tag = '整体平均千箱'
    sales_file_proc='湖北各商圈总销量TT+MT.xlsx'
    interpreter(time_tag, sale_tag, sales_file_proc, '湖北')



"""
Created on Sat Apr 21 07:29:46 2018

@author: jasonai
"""

from pmipy import log
import datetime


logger = log.createCustomLogger('root')

def execInfo(arg = True):
    if arg:
        def _deco(func):
            def wrapper(*arg, **kwargs):
                startTime = datetime.datetime.now()
                logger.info("start %s..."%func.__name__)
                func(*arg, **kwargs)
                endTime = datetime.datetime.now()
                secs = endTime - startTime
                logger.info("end {0}--> elapsed time: {1}".format(func.__name__, secs))
            return wrapper
    else:
        def _deco(func):
            return func
    return _deco


def checkResult(keyword, loopTime):
    def _deco(func):
        def wrapper(*arg, **kwargs):
            res2 = keyword
            time = 1
            while res2 == keyword and time <= loopTime:
                res = func(*arg, **kwargs)
                # 目前只考虑函数返回结果为list、字符串和数字这三种类型
                res2 = res[0] if type(res)==list else res 
                time += 1
            return res
        return wrapper

    return _deco

